@@ -19,7 +19,7 @@
  */
 
 #include <assert.h>
-#include "gdi_private.h"
+#include "ntgdi_private.h"
 #include "dibdrv.h"
 
 #include "wine/debug.h"
@@ -73,7 +73,7 @@ static BOOL brush_rect( dibdrv_physdev *pdev, dib_brush *brush, const RECT *rect
 
     if (!get_clipped_rects( &pdev->dib, rect, clip, &clipped_rects )) return TRUE;
     ret = brush->rects( pdev, brush, &pdev->dib, clipped_rects.count, clipped_rects.rects,
-                        &dc->brush_org, dc->ROPmode );
+                        &dc->attr->brush_org, dc->attr->rop_mode );
     free_clipped_rects( &clipped_rects );
     return ret;
 }
@@ -81,14 +81,14 @@ static BOOL brush_rect( dibdrv_physdev *pdev, dib_brush *brush, const RECT *rect
 /* paint a region with the brush (note: the region can be modified) */
 static BOOL brush_region( dibdrv_physdev *pdev, HRGN region )
 {
-    if (pdev->clip) CombineRgn( region, region, pdev->clip, RGN_AND );
+    if (pdev->clip) NtGdiCombineRgn( region, region, pdev->clip, RGN_AND );
     return brush_rect( pdev, &pdev->brush, NULL, region );
 }
 
 /* paint a region with the pen (note: the region can be modified) */
 static BOOL pen_region( dibdrv_physdev *pdev, HRGN region )
 {
-    if (pdev->clip) CombineRgn( region, region, pdev->clip, RGN_AND );
+    if (pdev->clip) NtGdiCombineRgn( region, region, pdev->clip, RGN_AND );
     return brush_rect( pdev, &pdev->pen_brush, NULL, region );
 }
 
@@ -100,7 +100,7 @@ static RECT get_device_rect( DC *dc, int left, int top, int right, int bottom, B
     rect.top    = top;
     rect.right  = right;
     rect.bottom = bottom;
-    if (rtl_correction && dc->layout & LAYOUT_RTL)
+    if (rtl_correction && dc->attr->layout & LAYOUT_RTL)
     {
         /* shift the rectangle so that the right border is included after mirroring */
         /* it would be more correct to do this after LPtoDP but that's not what Windows does */
@@ -345,11 +345,11 @@ static BOOL draw_arc( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
 
     if (extra_lines == -1)
     {
-        points[0] = dc->cur_pos;
+        points[0] = dc->attr->cur_pos;
         lp_to_dp( dc, points, 1 );
-        count = 1 + get_arc_points( dc->ArcDirection, &rect, pt[0], pt[1], points + 1 );
+        count = 1 + get_arc_points( dc->attr->arc_direction, &rect, pt[0], pt[1], points + 1 );
     }
-    else count = get_arc_points( dc->ArcDirection, &rect, pt[0], pt[1], points );
+    else count = get_arc_points( dc->attr->arc_direction, &rect, pt[0], pt[1], points );
 
     if (extra_lines == 2)
     {
@@ -363,7 +363,7 @@ static BOOL draw_arc( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
         return TRUE;
     }
 
-    if (pdev->pen_uses_region && !(outline = CreateRectRgn( 0, 0, 0, 0 )))
+    if (pdev->pen_uses_region && !(outline = NtGdiCreateRectRgn( 0, 0, 0, 0 )))
     {
         HeapFree( GetProcessHeap(), 0, points );
         return FALSE;
@@ -375,7 +375,7 @@ static BOOL draw_arc( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
         !(interior = create_polypolygon_region( points, &count, 1, WINDING, &rc )))
     {
         HeapFree( GetProcessHeap(), 0, points );
-        if (outline) DeleteObject( outline );
+        if (outline) NtGdiDeleteObjectApp( outline );
         return FALSE;
     }
 
@@ -383,7 +383,7 @@ static BOOL draw_arc( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
     if (interior && !outline)
     {
         ret = brush_region( pdev, interior );
-        DeleteObject( interior );
+        NtGdiDeleteObjectApp( interior );
         interior = 0;
     }
 
@@ -393,14 +393,14 @@ static BOOL draw_arc( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
 
     if (interior)
     {
-        CombineRgn( interior, interior, outline, RGN_DIFF );
+        NtGdiCombineRgn( interior, interior, outline, RGN_DIFF );
         ret = brush_region( pdev, interior );
-        DeleteObject( interior );
+        NtGdiDeleteObjectApp( interior );
     }
     if (outline)
     {
         if (ret) ret = pen_region( pdev, outline );
-        DeleteObject( outline );
+        NtGdiDeleteObjectApp( outline );
     }
     HeapFree( GetProcessHeap(), 0, points );
     return ret;
@@ -422,13 +422,13 @@ static BOOL stroke_and_fill_path( dibdrv_physdev *dev, BOOL stroke, BOOL fill )
     if (!(path = get_gdi_flat_path( dc, fill ? &interior : NULL ))) return FALSE;
     if (!(total = get_gdi_path_data( path, &points, &types ))) goto done;
 
-    if (stroke && dev->pen_uses_region) outline = CreateRectRgn( 0, 0, 0, 0 );
+    if (stroke && dev->pen_uses_region) outline = NtGdiCreateRectRgn( 0, 0, 0, 0 );
 
     /* if not using a region, paint the interior first so the outline can overlap it */
     if (interior && !outline)
     {
         ret = brush_region( dev, interior );
-        DeleteObject( interior );
+        NtGdiDeleteObjectApp( interior );
         interior = 0;
     }
 
@@ -458,14 +458,14 @@ static BOOL stroke_and_fill_path( dibdrv_physdev *dev, BOOL stroke, BOOL fill )
 
     if (interior)
     {
-        CombineRgn( interior, interior, outline, RGN_DIFF );
+        NtGdiCombineRgn( interior, interior, outline, RGN_DIFF );
         ret = brush_region( dev, interior );
-        DeleteObject( interior );
+        NtGdiDeleteObjectApp( interior );
     }
     if (outline)
     {
         if (ret) ret = pen_region( dev, outline );
-        DeleteObject( outline );
+        NtGdiDeleteObjectApp( outline );
     }
 
 done:
@@ -561,7 +561,7 @@ static struct cached_font *add_cached_font( DC *dc, HFONT hfont, UINT aa_flags )
     GetObjectW( hfont, sizeof(font.lf), &font.lf );
     font.xform = dc->xformWorld2Vport;
     font.xform.eDx = font.xform.eDy = 0;  /* unused, would break hashing */
-    if (dc->GraphicsMode == GM_COMPATIBLE)
+    if (dc->attr->graphics_mode == GM_COMPATIBLE)
     {
         font.lf.lfOrientation = font.lf.lfEscapement;
         if (font.xform.eM11 * font.xform.eM22 < 0)
@@ -666,7 +666,7 @@ static struct cached_glyph *get_cached_glyph( struct cached_font *font, UINT ind
  */
 static inline void get_text_bkgnd_masks( DC *dc, const dib_info *dib, rop_mask *mask )
 {
-    COLORREF bg = dc->backgroundColor;
+    COLORREF bg = dc->attr->background_color;
 
     mask->and = 0;
 
@@ -674,7 +674,7 @@ static inline void get_text_bkgnd_masks( DC *dc, const dib_info *dib, rop_mask *
         mask->xor = get_pixel_color( dc, dib, bg, FALSE );
     else
     {
-        COLORREF fg = dc->textColor;
+        COLORREF fg = dc->attr->text_color;
         mask->xor = get_pixel_color( dc, dib, fg, TRUE );
         if (fg != bg) mask->xor = ~mask->xor;
     }
@@ -823,7 +823,7 @@ static void render_string( DC *dc, dib_info *dib, struct cached_font *font, INT
     glyph_dib.bits.is_copy = FALSE;
     glyph_dib.bits.free    = NULL;
 
-    text_color = get_pixel_color( dc, dib, dc->textColor, TRUE );
+    text_color = get_pixel_color( dc, dib, dc->attr->text_color, TRUE );
 
     if (glyph_dib.bit_count == 32)
         intensity.gamma_ramp = dc->font_gamma_ramp;
@@ -1000,7 +1000,7 @@ BOOL CDECL dibdrv_Ellipse( PHYSDEV dev, INT left, INT top, INT right, INT bottom
 static inline BOOL is_interior( dib_info *dib, HRGN clip, int x, int y, DWORD pixel, UINT type)
 {
     /* the clip rgn stops the flooding */
-    if (clip && !PtInRegion( clip, x, y )) return FALSE;
+    if (clip && !NtGdiPtInRegion( clip, x, y )) return FALSE;
 
     if (type == FLOODFILLBORDER)
         return dib->funcs->get_pixel( dib, x, y ) != pixel;
@@ -1022,12 +1022,12 @@ static inline void do_next_row( dib_info *dib, HRGN clip, const RECT *row, int o
         if (is_interior( dib, clip, next.right, next.top, pixel, type)) next.right++;
         else
         {
-            if (next.left != next.right && !PtInRegion( rgn, next.left, next.top ))
+            if (next.left != next.right && !NtGdiPtInRegion( rgn, next.left, next.top ))
                 fill_row( dib, clip, &next, pixel, type, rgn );
             next.left = ++next.right;
         }
     }
-    if (next.left != next.right && !PtInRegion( rgn, next.left, next.top ))
+    if (next.left != next.right && !NtGdiPtInRegion( rgn, next.left, next.top ))
         fill_row( dib, clip, &next, pixel, type, rgn );
 }
 
@@ -1063,7 +1063,7 @@ BOOL CDECL dibdrv_ExtFloodFill( PHYSDEV dev, INT x, INT y, COLORREF color, UINT
 
     if (!is_interior( &pdev->dib, pdev->clip, x, y, pixel, type )) return FALSE;
 
-    if (!(rgn = CreateRectRgn( 0, 0, 0, 0 ))) return FALSE;
+    if (!(rgn = NtGdiCreateRectRgn( 0, 0, 0, 0 ))) return FALSE;
     row.left = x;
     row.right = x + 1;
     row.top = y;
@@ -1074,7 +1074,7 @@ BOOL CDECL dibdrv_ExtFloodFill( PHYSDEV dev, INT x, INT y, COLORREF color, UINT
     add_clipped_bounds( pdev, NULL, rgn );
     brush_region( pdev, rgn );
 
-    DeleteObject( rgn );
+    NtGdiDeleteObjectApp( rgn );
     return TRUE;
 }
 
@@ -1140,13 +1140,13 @@ BOOL CDECL dibdrv_LineTo( PHYSDEV dev, INT x, INT y )
     HRGN region = 0;
     BOOL ret;
 
-    pts[0] = dc->cur_pos;
+    pts[0] = dc->attr->cur_pos;
     pts[1].x = x;
     pts[1].y = y;
 
     lp_to_dp(dc, pts, 2);
 
-    if (pdev->pen_uses_region && !(region = CreateRectRgn( 0, 0, 0, 0 ))) return FALSE;
+    if (pdev->pen_uses_region && !(region = NtGdiCreateRectRgn( 0, 0, 0, 0 ))) return FALSE;
 
     reset_dash_origin(pdev);
 
@@ -1156,7 +1156,7 @@ BOOL CDECL dibdrv_LineTo( PHYSDEV dev, INT x, INT y )
     if (region)
     {
         ret = pen_region( pdev, region );
-        DeleteObject( region );
+        NtGdiDeleteObjectApp( region );
     }
     return ret;
 }
@@ -1203,7 +1203,7 @@ BOOL CDECL dibdrv_PatBlt( PHYSDEV dev, struct bitblt_coords *dst, DWORD rop )
         break;
     default:
         ret = brush->rects( pdev, brush, &pdev->dib, clipped_rects.count, clipped_rects.rects,
-                            &dc->brush_org, rop2 );
+                            &dc->attr->brush_org, rop2 );
         break;
     }
     free_clipped_rects( &clipped_rects );
@@ -1271,19 +1271,20 @@ BOOL CDECL dibdrv_PolyPolygon( PHYSDEV dev, const POINT *pt, const INT *counts,
 
     if (pdev->brush.style != BS_NULL &&
         get_dib_rect( &pdev->dib, &rc ) &&
-        !(interior = create_polypolygon_region( points, counts, polygons, dc->polyFillMode, &rc )))
+        !(interior = create_polypolygon_region( points, counts, polygons,
+                                                dc->attr->poly_fill_mode, &rc )))
     {
         ret = FALSE;
         goto done;
     }
 
-    if (pdev->pen_uses_region) outline = CreateRectRgn( 0, 0, 0, 0 );
+    if (pdev->pen_uses_region) outline = NtGdiCreateRectRgn( 0, 0, 0, 0 );
 
     /* if not using a region, paint the interior first so the outline can overlap it */
     if (interior && !outline)
     {
         ret = brush_region( pdev, interior );
-        DeleteObject( interior );
+        NtGdiDeleteObjectApp( interior );
         interior = 0;
     }
 
@@ -1297,14 +1298,14 @@ BOOL CDECL dibdrv_PolyPolygon( PHYSDEV dev, const POINT *pt, const INT *counts,
 
     if (interior)
     {
-        CombineRgn( interior, interior, outline, RGN_DIFF );
+        NtGdiCombineRgn( interior, interior, outline, RGN_DIFF );
         ret = brush_region( pdev, interior );
-        DeleteObject( interior );
+        NtGdiDeleteObjectApp( interior );
     }
     if (outline)
     {
         if (ret) ret = pen_region( pdev, outline );
-        DeleteObject( outline );
+        NtGdiDeleteObjectApp( outline );
     }
 
 done:
@@ -1339,7 +1340,7 @@ BOOL CDECL dibdrv_PolyPolyline( PHYSDEV dev, const POINT* pt, const DWORD* count
     memcpy( points, pt, total * sizeof(*pt) );
     lp_to_dp( dc, points, total );
 
-    if (pdev->pen_uses_region && !(outline = CreateRectRgn( 0, 0, 0, 0 )))
+    if (pdev->pen_uses_region && !(outline = NtGdiCreateRectRgn( 0, 0, 0, 0 )))
     {
         ret = FALSE;
         goto done;
@@ -1356,7 +1357,7 @@ BOOL CDECL dibdrv_PolyPolyline( PHYSDEV dev, const POINT* pt, const DWORD* count
     if (outline)
     {
         ret = pen_region( pdev, outline );
-        DeleteObject( outline );
+        NtGdiDeleteObjectApp( outline );
     }
 
 done:
@@ -1364,27 +1365,6 @@ done:
     return ret;
 }
 
-/***********************************************************************
- *           dibdrv_Polygon
- */
-BOOL CDECL dibdrv_Polygon( PHYSDEV dev, const POINT *pt, INT count )
-{
-    INT counts[1] = { count };
-
-    return dibdrv_PolyPolygon( dev, pt, counts, 1 );
-}
-
-/***********************************************************************
- *           dibdrv_Polyline
- */
-BOOL CDECL dibdrv_Polyline( PHYSDEV dev, const POINT* pt, INT count )
-{
-    DWORD counts[1] = { count };
-
-    if (count < 0) return FALSE;
-    return dibdrv_PolyPolyline( dev, pt, counts, 1 );
-}
-
 /***********************************************************************
  *           dibdrv_Rectangle
  */
@@ -1399,24 +1379,25 @@ BOOL CDECL dibdrv_Rectangle( PHYSDEV dev, INT left, INT top, INT right, INT bott
 
     TRACE("(%p, %d, %d, %d, %d)\n", dev, left, top, right, bottom);
 
-    if (dc->GraphicsMode == GM_ADVANCED)
+    if (dc->attr->graphics_mode == GM_ADVANCED)
     {
+        const INT count = 4;
         pts[0].x = pts[3].x = left;
         pts[0].y = pts[1].y = top;
         pts[1].x = pts[2].x = right;
         pts[2].y = pts[3].y = bottom;
-        return dibdrv_Polygon( dev, pts, 4 );
+        return dibdrv_PolyPolygon( dev, pts, &count, 1 );
     }
 
     if (!get_pen_device_rect( dc, pdev, &rect, left, top, right, bottom )) return TRUE;
 
-    if (pdev->pen_uses_region && !(outline = CreateRectRgn( 0, 0, 0, 0 ))) return FALSE;
+    if (pdev->pen_uses_region && !(outline = NtGdiCreateRectRgn( 0, 0, 0, 0 ))) return FALSE;
 
     rect.right--;
     rect.bottom--;
     reset_dash_origin(pdev);
 
-    if (dc->ArcDirection == AD_CLOCKWISE)
+    if (dc->attr->arc_direction == AD_CLOCKWISE)
     {
         /* 4 pts going clockwise starting from bottom-right */
         pts[0].x = pts[3].x = rect.right;
@@ -1442,12 +1423,12 @@ BOOL CDECL dibdrv_Rectangle( PHYSDEV dev, INT left, INT top, INT right, INT bott
         {
             HRGN interior = CreateRectRgnIndirect( &rect );
 
-            CombineRgn( interior, interior, outline, RGN_DIFF );
+            NtGdiCombineRgn( interior, interior, outline, RGN_DIFF );
             brush_region( pdev, interior );
-            DeleteObject( interior );
+            NtGdiDeleteObjectApp( interior );
         }
         ret = pen_region( pdev, outline );
-        DeleteObject( outline );
+        NtGdiDeleteObjectApp( outline );
     }
     else
     {
@@ -1488,18 +1469,18 @@ BOOL CDECL dibdrv_RoundRect( PHYSDEV dev, INT left, INT top, INT right, INT bott
     points = HeapAlloc( GetProcessHeap(), 0, (ellipse_width + ellipse_height) * 2 * sizeof(*points) );
     if (!points) return FALSE;
 
-    if (pdev->pen_uses_region && !(outline = CreateRectRgn( 0, 0, 0, 0 )))
+    if (pdev->pen_uses_region && !(outline = NtGdiCreateRectRgn( 0, 0, 0, 0 )))
     {
         HeapFree( GetProcessHeap(), 0, points );
         return FALSE;
     }
 
     if (pdev->brush.style != BS_NULL &&
-        !(interior = CreateRoundRectRgn( rect.left, rect.top, rect.right + 1, rect.bottom + 1,
-                                         ellipse_width, ellipse_height )))
+        !(interior = NtGdiCreateRoundRectRgn( rect.left, rect.top, rect.right + 1, rect.bottom + 1,
+                                              ellipse_width, ellipse_height )))
     {
         HeapFree( GetProcessHeap(), 0, points );
-        if (outline) DeleteObject( outline );
+        if (outline) NtGdiDeleteObjectApp( outline );
         return FALSE;
     }
 
@@ -1507,13 +1488,13 @@ BOOL CDECL dibdrv_RoundRect( PHYSDEV dev, INT left, INT top, INT right, INT bott
     if (interior && !outline)
     {
         ret = brush_region( pdev, interior );
-        DeleteObject( interior );
+        NtGdiDeleteObjectApp( interior );
         interior = 0;
     }
 
     count = ellipse_first_quadrant( ellipse_width, ellipse_height, points );
 
-    if (dc->ArcDirection == AD_CLOCKWISE)
+    if (dc->attr->arc_direction == AD_CLOCKWISE)
     {
         for (i = 0; i < count; i++)
         {
@@ -1560,14 +1541,14 @@ BOOL CDECL dibdrv_RoundRect( PHYSDEV dev, INT left, INT top, INT right, INT bott
 
     if (interior)
     {
-        CombineRgn( interior, interior, outline, RGN_DIFF );
+        NtGdiCombineRgn( interior, interior, outline, RGN_DIFF );
         ret = brush_region( pdev, interior );
-        DeleteObject( interior );
+        NtGdiDeleteObjectApp( interior );
     }
     if (outline)
     {
         if (ret) ret = pen_region( pdev, outline );
-        DeleteObject( outline );
+        NtGdiDeleteObjectApp( outline );
     }
     HeapFree( GetProcessHeap(), 0, points );
     return ret;
@@ -1610,7 +1591,7 @@ COLORREF CDECL dibdrv_SetPixel( PHYSDEV dev, INT x, INT y, COLORREF color )
     color = pdev->dib.funcs->pixel_to_colorref( &pdev->dib, pixel );
 
     if (!get_clipped_rects( &pdev->dib, &rect, pdev->clip, &clipped_rects )) return color;
-    fill_with_pixel( dc, &pdev->dib, pixel, clipped_rects.count, clipped_rects.rects, dc->ROPmode );
+    fill_with_pixel( dc, &pdev->dib, pixel, clipped_rects.count, clipped_rects.rects, dc->attr->rop_mode );
     free_clipped_rects( &clipped_rects );
     return color;
 }
