@@ -29,24 +29,16 @@
 #include "winbase.h"
 #include "wingdi.h"
 #include "winreg.h"
-#include "ddrawgdi.h"
 #include "wine/winbase16.h"
 #include "winuser.h"
-#include "winternl.h"
-#include "initguid.h"
-#include "devguid.h"
-#include "setupapi.h"
-#include "ddk/d3dkmthk.h"
 
-#include "gdi_private.h"
+#include "ntgdi_private.h"
 #include "wine/list.h"
 #include "wine/debug.h"
 #include "wine/heap.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(driver);
 
-DEFINE_DEVPROPKEY(DEVPROPKEY_GPU_LUID, 0x60b193cb, 0x5276, 0x4d0f, 0x96, 0xfc, 0xf1, 0x73, 0xab, 0xad, 0x3e, 0xc6, 2);
-
 struct graphics_driver
 {
     struct list                entry;
@@ -163,20 +155,6 @@ BOOL is_display_device( LPCWSTR name )
     return TRUE;
 }
 
-static HANDLE get_display_device_init_mutex( void )
-{
-    HANDLE mutex = CreateMutexW( NULL, FALSE, L"display_device_init" );
-
-    WaitForSingleObject( mutex, INFINITE );
-    return mutex;
-}
-
-static void release_display_device_init_mutex( HANDLE mutex )
-{
-    ReleaseMutex( mutex );
-    CloseHandle( mutex );
-}
-
 #ifdef __i386__
 static const WCHAR printer_env[] = L"w32x86";
 #elif defined __x86_64__
@@ -247,7 +225,7 @@ done:
 
 
 /***********************************************************************
- *           __wine_set_display_driver    (GDI32.@)
+ *           __wine_set_display_driver    (win32u.@)
  */
 void CDECL __wine_set_display_driver( HMODULE module )
 {
@@ -359,11 +337,6 @@ static BOOL CDECL nulldrv_EnumFonts( PHYSDEV dev, LOGFONTW *logfont, FONTENUMPRO
     return TRUE;
 }
 
-static INT CDECL nulldrv_EnumICMProfiles( PHYSDEV dev, ICMENUMPROCW func, LPARAM lparam )
-{
-    return -1;
-}
-
 static INT CDECL nulldrv_ExtDeviceMode( LPSTR buffer, HWND hwnd, DEVMODEA *output, LPSTR device,
                                         LPSTR port, DEVMODEA *input, LPSTR profile, DWORD mode )
 {
@@ -386,17 +359,13 @@ static BOOL CDECL nulldrv_FontIsLinked( PHYSDEV dev )
     return FALSE;
 }
 
-static BOOL CDECL nulldrv_GdiComment( PHYSDEV dev, UINT size, const BYTE *data )
-{
-    return FALSE;
-}
-
 static UINT CDECL nulldrv_GetBoundsRect( PHYSDEV dev, RECT *rect, UINT flags )
 {
     return DCB_RESET;
 }
 
-static BOOL CDECL nulldrv_GetCharABCWidths( PHYSDEV dev, UINT first, UINT last, LPABC abc )
+static BOOL CDECL nulldrv_GetCharABCWidths( PHYSDEV dev, UINT first, UINT count,
+                                            WCHAR *chars, ABC *abc )
 {
     return FALSE;
 }
@@ -406,7 +375,8 @@ static BOOL CDECL nulldrv_GetCharABCWidthsI( PHYSDEV dev, UINT first, UINT count
     return FALSE;
 }
 
-static BOOL CDECL nulldrv_GetCharWidth( PHYSDEV dev, UINT first, UINT last, INT *buffer )
+static BOOL CDECL nulldrv_GetCharWidth( PHYSDEV dev, UINT first, UINT count,
+                                        const WCHAR *chars, INT *buffer )
 {
     return FALSE;
 }
@@ -460,7 +430,24 @@ static INT CDECL nulldrv_GetDeviceCaps( PHYSDEV dev, INT cap )
 
         return pGetSystemMetrics ? pGetSystemMetrics( SM_CYSCREEN ) : 480;
     }
-    case BITSPIXEL:       return 32;
+    case BITSPIXEL:
+    {
+        DEVMODEW devmode;
+        WCHAR *display;
+        DC *dc;
+
+        if (GetDeviceCaps( dev->hdc, TECHNOLOGY ) == DT_RASDISPLAY && pEnumDisplaySettingsW)
+        {
+            dc = get_nulldrv_dc( dev );
+            display = dc->display[0] ? dc->display : NULL;
+            memset( &devmode, 0, sizeof(devmode) );
+            devmode.dmSize = sizeof(devmode);
+            if (pEnumDisplaySettingsW( display, ENUM_CURRENT_SETTINGS, &devmode )
+                && devmode.dmFields & DM_BITSPERPEL && devmode.dmBitsPerPel)
+                return devmode.dmBitsPerPel;
+        }
+        return 32;
+    }
     case PLANES:          return 1;
     case NUMBRUSHES:      return -1;
     case NUMPENS:         return -1;
@@ -591,7 +578,7 @@ static DWORD CDECL nulldrv_GetGlyphOutline( PHYSDEV dev, UINT ch, UINT format, L
     return GDI_ERROR;
 }
 
-static BOOL CDECL nulldrv_GetICMProfile( PHYSDEV dev, LPDWORD size, LPWSTR filename )
+static BOOL CDECL nulldrv_GetICMProfile( PHYSDEV dev, BOOL allow_default, LPDWORD size, LPWSTR filename )
 {
     return FALSE;
 }
@@ -686,21 +673,6 @@ static BOOL CDECL nulldrv_PolyPolyline( PHYSDEV dev, const POINT *points, const
     return TRUE;
 }
 
-static BOOL CDECL nulldrv_Polygon( PHYSDEV dev, const POINT *points, INT count )
-{
-    INT counts[1] = { count };
-
-    return PolyPolygon( dev->hdc, points, counts, 1 );
-}
-
-static BOOL CDECL nulldrv_Polyline( PHYSDEV dev, const POINT *points, INT count )
-{
-    DWORD counts[1] = { count };
-
-    if (count < 0) return FALSE;
-    return PolyPolyline( dev->hdc, points, counts, 1 );
-}
-
 static DWORD CDECL nulldrv_PutImage( PHYSDEV dev, HRGN clip, BITMAPINFO *info,
                                      const struct gdi_image_bits *bits, struct bitblt_coords *src,
                                      struct bitblt_coords *dst, DWORD rop )
@@ -723,9 +695,9 @@ static BOOL CDECL nulldrv_Rectangle( PHYSDEV dev, INT left, INT top, INT right,
     return TRUE;
 }
 
-static HDC CDECL nulldrv_ResetDC( PHYSDEV dev, const DEVMODEW *devmode )
+static BOOL CDECL nulldrv_ResetDC( PHYSDEV dev, const DEVMODEW *devmode )
 {
-    return 0;
+    return FALSE;
 }
 
 static BOOL CDECL nulldrv_RoundRect( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
@@ -749,31 +721,16 @@ static HFONT CDECL nulldrv_SelectFont( PHYSDEV dev, HFONT font, UINT *aa_flags )
     return font;
 }
 
-static HPALETTE CDECL nulldrv_SelectPalette( PHYSDEV dev, HPALETTE palette, BOOL bkgnd )
-{
-    return palette;
-}
-
 static HPEN CDECL nulldrv_SelectPen( PHYSDEV dev, HPEN pen, const struct brush_pattern *pattern )
 {
     return pen;
 }
 
-static INT CDECL nulldrv_SetArcDirection( PHYSDEV dev, INT dir )
-{
-    return dir;
-}
-
 static COLORREF CDECL nulldrv_SetBkColor( PHYSDEV dev, COLORREF color )
 {
     return color;
 }
 
-static INT CDECL nulldrv_SetBkMode( PHYSDEV dev, INT mode )
-{
-    return mode;
-}
-
 static UINT CDECL nulldrv_SetBoundsRect( PHYSDEV dev, RECT *rect, UINT flags )
 {
     return DCB_RESET;
@@ -793,67 +750,22 @@ static void CDECL nulldrv_SetDeviceClipping( PHYSDEV dev, HRGN rgn )
 {
 }
 
-static DWORD CDECL nulldrv_SetLayout( PHYSDEV dev, DWORD layout )
-{
-    return layout;
-}
-
 static BOOL CDECL nulldrv_SetDeviceGammaRamp( PHYSDEV dev, void *ramp )
 {
     SetLastError( ERROR_INVALID_PARAMETER );
     return FALSE;
 }
 
-static DWORD CDECL nulldrv_SetMapperFlags( PHYSDEV dev, DWORD flags )
-{
-    return flags;
-}
-
 static COLORREF CDECL nulldrv_SetPixel( PHYSDEV dev, INT x, INT y, COLORREF color )
 {
     return color;
 }
 
-static INT CDECL nulldrv_SetPolyFillMode( PHYSDEV dev, INT mode )
-{
-    return mode;
-}
-
-static INT CDECL nulldrv_SetROP2( PHYSDEV dev, INT rop )
-{
-    return rop;
-}
-
-static INT CDECL nulldrv_SetRelAbs( PHYSDEV dev, INT mode )
-{
-    return mode;
-}
-
-static INT CDECL nulldrv_SetStretchBltMode( PHYSDEV dev, INT mode )
-{
-    return mode;
-}
-
-static UINT CDECL nulldrv_SetTextAlign( PHYSDEV dev, UINT align )
-{
-    return align;
-}
-
-static INT CDECL nulldrv_SetTextCharacterExtra( PHYSDEV dev, INT extra )
-{
-    return extra;
-}
-
 static COLORREF CDECL nulldrv_SetTextColor( PHYSDEV dev, COLORREF color )
 {
     return color;
 }
 
-static BOOL CDECL nulldrv_SetTextJustification( PHYSDEV dev, INT extra, INT breaks )
-{
-    return TRUE;
-}
-
 static INT CDECL nulldrv_StartDoc( PHYSDEV dev, const DOCINFOW *info )
 {
     return 0;
@@ -911,19 +823,14 @@ const struct gdi_dc_funcs null_driver =
     nulldrv_EndPage,                    /* pEndPage */
     nulldrv_EndPath,                    /* pEndPath */
     nulldrv_EnumFonts,                  /* pEnumFonts */
-    nulldrv_EnumICMProfiles,            /* pEnumICMProfiles */
-    nulldrv_ExcludeClipRect,            /* pExcludeClipRect */
     nulldrv_ExtDeviceMode,              /* pExtDeviceMode */
     nulldrv_ExtEscape,                  /* pExtEscape */
     nulldrv_ExtFloodFill,               /* pExtFloodFill */
-    nulldrv_ExtSelectClipRgn,           /* pExtSelectClipRgn */
     nulldrv_ExtTextOut,                 /* pExtTextOut */
     nulldrv_FillPath,                   /* pFillPath */
     nulldrv_FillRgn,                    /* pFillRgn */
-    nulldrv_FlattenPath,                /* pFlattenPath */
     nulldrv_FontIsLinked,               /* pFontIsLinked */
     nulldrv_FrameRgn,                   /* pFrameRgn */
-    nulldrv_GdiComment,                 /* pGdiComment */
     nulldrv_GetBoundsRect,              /* pGetBoundsRect */
     nulldrv_GetCharABCWidths,           /* pGetCharABCWidths */
     nulldrv_GetCharABCWidthsI,          /* pGetCharABCWidthsI */
@@ -949,14 +856,9 @@ const struct gdi_dc_funcs null_driver =
     nulldrv_GetTextFace,                /* pGetTextFace */
     nulldrv_GetTextMetrics,             /* pGetTextMetrics */
     nulldrv_GradientFill,               /* pGradientFill */
-    nulldrv_IntersectClipRect,          /* pIntersectClipRect */
     nulldrv_InvertRgn,                  /* pInvertRgn */
     nulldrv_LineTo,                     /* pLineTo */
-    nulldrv_ModifyWorldTransform,       /* pModifyWorldTransform */
     nulldrv_MoveTo,                     /* pMoveTo */
-    nulldrv_OffsetClipRgn,              /* pOffsetClipRgn */
-    nulldrv_OffsetViewportOrgEx,        /* pOffsetViewportOrg */
-    nulldrv_OffsetWindowOrgEx,          /* pOffsetWindowOrg */
     nulldrv_PaintRgn,                   /* pPaintRgn */
     nulldrv_PatBlt,                     /* pPatBlt */
     nulldrv_Pie,                        /* pPie */
@@ -965,51 +867,26 @@ const struct gdi_dc_funcs null_driver =
     nulldrv_PolyDraw,                   /* pPolyDraw */
     nulldrv_PolyPolygon,                /* pPolyPolygon */
     nulldrv_PolyPolyline,               /* pPolyPolyline */
-    nulldrv_Polygon,                    /* pPolygon */
-    nulldrv_Polyline,                   /* pPolyline */
     nulldrv_PolylineTo,                 /* pPolylineTo */
     nulldrv_PutImage,                   /* pPutImage */
     nulldrv_RealizeDefaultPalette,      /* pRealizeDefaultPalette */
     nulldrv_RealizePalette,             /* pRealizePalette */
     nulldrv_Rectangle,                  /* pRectangle */
     nulldrv_ResetDC,                    /* pResetDC */
-    nulldrv_RestoreDC,                  /* pRestoreDC */
     nulldrv_RoundRect,                  /* pRoundRect */
-    nulldrv_SaveDC,                     /* pSaveDC */
-    nulldrv_ScaleViewportExtEx,         /* pScaleViewportExt */
-    nulldrv_ScaleWindowExtEx,           /* pScaleWindowExt */
     nulldrv_SelectBitmap,               /* pSelectBitmap */
     nulldrv_SelectBrush,                /* pSelectBrush */
-    nulldrv_SelectClipPath,             /* pSelectClipPath */
     nulldrv_SelectFont,                 /* pSelectFont */
-    nulldrv_SelectPalette,              /* pSelectPalette */
     nulldrv_SelectPen,                  /* pSelectPen */
-    nulldrv_SetArcDirection,            /* pSetArcDirection */
     nulldrv_SetBkColor,                 /* pSetBkColor */
-    nulldrv_SetBkMode,                  /* pSetBkMode */
     nulldrv_SetBoundsRect,              /* pSetBoundsRect */
     nulldrv_SetDCBrushColor,            /* pSetDCBrushColor */
     nulldrv_SetDCPenColor,              /* pSetDCPenColor */
     nulldrv_SetDIBitsToDevice,          /* pSetDIBitsToDevice */
     nulldrv_SetDeviceClipping,          /* pSetDeviceClipping */
     nulldrv_SetDeviceGammaRamp,         /* pSetDeviceGammaRamp */
-    nulldrv_SetLayout,                  /* pSetLayout */
-    nulldrv_SetMapMode,                 /* pSetMapMode */
-    nulldrv_SetMapperFlags,             /* pSetMapperFlags */
     nulldrv_SetPixel,                   /* pSetPixel */
-    nulldrv_SetPolyFillMode,            /* pSetPolyFillMode */
-    nulldrv_SetROP2,                    /* pSetROP2 */
-    nulldrv_SetRelAbs,                  /* pSetRelAbs */
-    nulldrv_SetStretchBltMode,          /* pSetStretchBltMode */
-    nulldrv_SetTextAlign,               /* pSetTextAlign */
-    nulldrv_SetTextCharacterExtra,      /* pSetTextCharacterExtra */
     nulldrv_SetTextColor,               /* pSetTextColor */
-    nulldrv_SetTextJustification,       /* pSetTextJustification */
-    nulldrv_SetViewportExtEx,           /* pSetViewportExt */
-    nulldrv_SetViewportOrgEx,           /* pSetViewportOrg */
-    nulldrv_SetWindowExtEx,             /* pSetWindowExt */
-    nulldrv_SetWindowOrgEx,             /* pSetWindowOrg */
-    nulldrv_SetWorldTransform,          /* pSetWorldTransform */
     nulldrv_StartDoc,                   /* pStartDoc */
     nulldrv_StartPage,                  /* pStartPage */
     nulldrv_StretchBlt,                 /* pStretchBlt */
@@ -1017,7 +894,6 @@ const struct gdi_dc_funcs null_driver =
     nulldrv_StrokeAndFillPath,          /* pStrokeAndFillPath */
     nulldrv_StrokePath,                 /* pStrokePath */
     nulldrv_UnrealizePalette,           /* pUnrealizePalette */
-    nulldrv_WidenPath,                  /* pWidenPath */
     nulldrv_D3DKMTCheckVidPnExclusiveOwnership, /* pD3DKMTCheckVidPnExclusiveOwnership */
     nulldrv_D3DKMTSetVidPnSourceOwner,  /* pD3DKMTSetVidPnSourceOwner */
     nulldrv_wine_get_wgl_driver,        /* wine_get_wgl_driver */
@@ -1059,56 +935,6 @@ BOOL DRIVER_GetDriverName( LPCWSTR device, LPWSTR driver, DWORD size )
 }
 
 
-/***********************************************************************
- *           GdiConvertToDevmodeW    (GDI32.@)
- */
-DEVMODEW * WINAPI GdiConvertToDevmodeW(const DEVMODEA *dmA)
-{
-    DEVMODEW *dmW;
-    WORD dmW_size, dmA_size;
-
-    dmA_size = dmA->dmSize;
-
-    /* this is the minimal dmSize that XP accepts */
-    if (dmA_size < FIELD_OFFSET(DEVMODEA, dmFields))
-        return NULL;
-
-    if (dmA_size > sizeof(DEVMODEA))
-        dmA_size = sizeof(DEVMODEA);
-
-    dmW_size = dmA_size + CCHDEVICENAME;
-    if (dmA_size >= FIELD_OFFSET(DEVMODEA, dmFormName) + CCHFORMNAME)
-        dmW_size += CCHFORMNAME;
-
-    dmW = HeapAlloc(GetProcessHeap(), 0, dmW_size + dmA->dmDriverExtra);
-    if (!dmW) return NULL;
-
-    MultiByteToWideChar(CP_ACP, 0, (const char*) dmA->dmDeviceName, -1,
-                                   dmW->dmDeviceName, CCHDEVICENAME);
-    /* copy slightly more, to avoid long computations */
-    memcpy(&dmW->dmSpecVersion, &dmA->dmSpecVersion, dmA_size - CCHDEVICENAME);
-
-    if (dmA_size >= FIELD_OFFSET(DEVMODEA, dmFormName) + CCHFORMNAME)
-    {
-        if (dmA->dmFields & DM_FORMNAME)
-            MultiByteToWideChar(CP_ACP, 0, (const char*) dmA->dmFormName, -1,
-                                       dmW->dmFormName, CCHFORMNAME);
-        else
-            dmW->dmFormName[0] = 0;
-
-        if (dmA_size > FIELD_OFFSET(DEVMODEA, dmLogPixels))
-            memcpy(&dmW->dmLogPixels, &dmA->dmLogPixels, dmA_size - FIELD_OFFSET(DEVMODEA, dmLogPixels));
-    }
-
-    if (dmA->dmDriverExtra)
-        memcpy((char *)dmW + dmW_size, (const char *)dmA + dmA_size, dmA->dmDriverExtra);
-
-    dmW->dmSize = dmW_size;
-
-    return dmW;
-}
-
-
 /*****************************************************************************
  *      @ [GDI32.100]
  *
@@ -1243,123 +1069,13 @@ DWORD WINAPI GDI_CallDeviceCapabilities16( LPCSTR lpszDevice, LPCSTR lpszPort,
 }
 
 
-/************************************************************************
- *             Escape  [GDI32.@]
- */
-INT WINAPI Escape( HDC hdc, INT escape, INT in_count, LPCSTR in_data, LPVOID out_data )
-{
-    INT ret;
-    POINT *pt;
-
-    switch (escape)
-    {
-    case ABORTDOC:
-        return AbortDoc( hdc );
-
-    case ENDDOC:
-        return EndDoc( hdc );
-
-    case GETPHYSPAGESIZE:
-        pt = out_data;
-        pt->x = GetDeviceCaps( hdc, PHYSICALWIDTH );
-        pt->y = GetDeviceCaps( hdc, PHYSICALHEIGHT );
-        return 1;
-
-    case GETPRINTINGOFFSET:
-        pt = out_data;
-        pt->x = GetDeviceCaps( hdc, PHYSICALOFFSETX );
-        pt->y = GetDeviceCaps( hdc, PHYSICALOFFSETY );
-        return 1;
-
-    case GETSCALINGFACTOR:
-        pt = out_data;
-        pt->x = GetDeviceCaps( hdc, SCALINGFACTORX );
-        pt->y = GetDeviceCaps( hdc, SCALINGFACTORY );
-        return 1;
-
-    case NEWFRAME:
-        return EndPage( hdc );
-
-    case SETABORTPROC:
-        return SetAbortProc( hdc, (ABORTPROC)in_data );
-
-    case STARTDOC:
-        {
-            DOCINFOA doc;
-            char *name = NULL;
-
-            /* in_data may not be 0 terminated so we must copy it */
-            if (in_data)
-            {
-                name = HeapAlloc( GetProcessHeap(), 0, in_count+1 );
-                memcpy( name, in_data, in_count );
-                name[in_count] = 0;
-            }
-            /* out_data is actually a pointer to the DocInfo structure and used as
-             * a second input parameter */
-            if (out_data) doc = *(DOCINFOA *)out_data;
-            else
-            {
-                doc.cbSize = sizeof(doc);
-                doc.lpszOutput = NULL;
-                doc.lpszDatatype = NULL;
-                doc.fwType = 0;
-            }
-            doc.lpszDocName = name;
-            ret = StartDocA( hdc, &doc );
-            HeapFree( GetProcessHeap(), 0, name );
-            if (ret > 0) ret = StartPage( hdc );
-            return ret;
-        }
-
-    case QUERYESCSUPPORT:
-        {
-            DWORD code;
-
-            if (in_count < sizeof(SHORT)) return 0;
-            code = (in_count < sizeof(DWORD)) ? *(const USHORT *)in_data : *(const DWORD *)in_data;
-            switch (code)
-            {
-            case ABORTDOC:
-            case ENDDOC:
-            case GETPHYSPAGESIZE:
-            case GETPRINTINGOFFSET:
-            case GETSCALINGFACTOR:
-            case NEWFRAME:
-            case QUERYESCSUPPORT:
-            case SETABORTPROC:
-            case STARTDOC:
-                return TRUE;
-            }
-            break;
-        }
-    }
-
-    /* if not handled internally, pass it to the driver */
-    return ExtEscape( hdc, escape, in_count, in_data, 0, out_data );
-}
-
-
 /******************************************************************************
- *		ExtEscape	[GDI32.@]
+ *		NtGdiExtEscape   (win32u.@)
  *
  * Access capabilities of a particular device that are not available through GDI.
- *
- * PARAMS
- *    hdc         [I] Handle to device context
- *    nEscape     [I] Escape function
- *    cbInput     [I] Number of bytes in input structure
- *    lpszInData  [I] Pointer to input structure
- *    cbOutput    [I] Number of bytes in output structure
- *    lpszOutData [O] Pointer to output structure
- *
- * RETURNS
- *    Success: >0
- *    Not implemented: 0
- *    Failure: <0
  */
-INT WINAPI ExtEscape( HDC hdc, INT nEscape, INT cbInput, LPCSTR lpszInData,
-                      INT cbOutput, LPSTR lpszOutData )
+INT WINAPI NtGdiExtEscape( HDC hdc, WCHAR *driver, int driver_id, INT escape, INT input_size,
+                           const char *input, INT output_size, char *output )
 {
     PHYSDEV physdev;
     INT ret;
@@ -1368,70 +1084,34 @@ INT WINAPI ExtEscape( HDC hdc, INT nEscape, INT cbInput, LPCSTR lpszInData,
     if (!dc) return 0;
     update_dc( dc );
     physdev = GET_DC_PHYSDEV( dc, pExtEscape );
-    ret = physdev->funcs->pExtEscape( physdev, nEscape, cbInput, lpszInData, cbOutput, lpszOutData );
+    ret = physdev->funcs->pExtEscape( physdev, escape, input_size, input, output_size, output );
     release_dc_ptr( dc );
     return ret;
 }
 
 
-/*******************************************************************
- *      DrawEscape [GDI32.@]
- *
- *
- */
-INT WINAPI DrawEscape(HDC hdc, INT nEscape, INT cbInput, LPCSTR lpszInData)
-{
-    FIXME("DrawEscape, stub\n");
-    return 0;
-}
-
-/*******************************************************************
- *      NamedEscape [GDI32.@]
- */
-INT WINAPI NamedEscape( HDC hdc, LPCWSTR pDriver, INT nEscape, INT cbInput, LPCSTR lpszInData,
-                        INT cbOutput, LPSTR lpszOutData )
-{
-    FIXME("(%p, %s, %d, %d, %p, %d, %p)\n",
-          hdc, wine_dbgstr_w(pDriver), nEscape, cbInput, lpszInData, cbOutput,
-          lpszOutData);
-    return 0;
-}
-
-/*******************************************************************
- *      DdQueryDisplaySettingsUniqueness [GDI32.@]
- *      GdiEntry13                       [GDI32.@]
- */
-ULONG WINAPI DdQueryDisplaySettingsUniqueness(VOID)
-{
-    static int warn_once;
-
-    if (!warn_once++)
-        FIXME("stub\n");
-    return 0;
-}
-
 /******************************************************************************
- *		D3DKMTOpenAdapterFromHdc [GDI32.@]
+ *           NtGdiDdDDIOpenAdapterFromHdc    (win32u.@)
  */
-NTSTATUS WINAPI D3DKMTOpenAdapterFromHdc( void *pData )
+NTSTATUS WINAPI NtGdiDdDDIOpenAdapterFromHdc( void *pData )
 {
     FIXME("(%p): stub\n", pData);
     return STATUS_NO_MEMORY;
 }
 
 /******************************************************************************
- *		D3DKMTEscape [GDI32.@]
+ *           NtGdiDdDDIEscape    (win32u.@)
  */
-NTSTATUS WINAPI D3DKMTEscape( const void *pData )
+NTSTATUS WINAPI NtGdiDdDDIEscape( const void *pData )
 {
     FIXME("(%p): stub\n", pData);
     return STATUS_NO_MEMORY;
 }
 
 /******************************************************************************
- *		D3DKMTCloseAdapter [GDI32.@]
+ *           NtGdiDdDDICloseAdapter    (win32u.@)
  */
-NTSTATUS WINAPI D3DKMTCloseAdapter( const D3DKMT_CLOSEADAPTER *desc )
+NTSTATUS WINAPI NtGdiDdDDICloseAdapter( const D3DKMT_CLOSEADAPTER *desc )
 {
     NTSTATUS status = STATUS_INVALID_PARAMETER;
     struct d3dkmt_adapter *adapter;
@@ -1458,91 +1138,26 @@ NTSTATUS WINAPI D3DKMTCloseAdapter( const D3DKMT_CLOSEADAPTER *desc )
 }
 
 /******************************************************************************
- *		D3DKMTOpenAdapterFromGdiDisplayName [GDI32.@]
+ *           NtGdiDdDDIOpenAdapterFromLuid    (win32u.@)
  */
-NTSTATUS WINAPI D3DKMTOpenAdapterFromGdiDisplayName( D3DKMT_OPENADAPTERFROMGDIDISPLAYNAME *desc )
+NTSTATUS WINAPI NtGdiDdDDIOpenAdapterFromLuid( D3DKMT_OPENADAPTERFROMLUID *desc )
 {
-    WCHAR *end, key_nameW[MAX_PATH], bufferW[MAX_PATH];
-    HDEVINFO devinfo = INVALID_HANDLE_VALUE;
-    NTSTATUS status = STATUS_UNSUCCESSFUL;
     static D3DKMT_HANDLE handle_start = 0;
     struct d3dkmt_adapter *adapter;
-    SP_DEVINFO_DATA device_data;
-    DWORD size, state_flags;
-    DEVPROPTYPE type;
-    HANDLE mutex;
-    LUID luid;
-    int index;
 
-    TRACE("(%p)\n", desc);
-
-    if (!desc)
-        return STATUS_UNSUCCESSFUL;
-
-    TRACE("DeviceName: %s\n", wine_dbgstr_w( desc->DeviceName ));
-    if (wcsnicmp( desc->DeviceName, L"\\\\.\\DISPLAY", lstrlenW(L"\\\\.\\DISPLAY") ))
-        return STATUS_UNSUCCESSFUL;
-
-    index = wcstol( desc->DeviceName + lstrlenW(L"\\\\.\\DISPLAY"), &end, 10 ) - 1;
-    if (*end)
-        return STATUS_UNSUCCESSFUL;
-
-    adapter = heap_alloc( sizeof( *adapter ) );
-    if (!adapter)
-        return STATUS_NO_MEMORY;
-
-    /* Get adapter LUID from SetupAPI */
-    mutex = get_display_device_init_mutex();
-
-    size = sizeof( bufferW );
-    swprintf( key_nameW, MAX_PATH, L"\\Device\\Video%d", index );
-    if (RegGetValueW( HKEY_LOCAL_MACHINE, L"HARDWARE\\DEVICEMAP\\VIDEO", key_nameW, RRF_RT_REG_SZ, NULL, bufferW, &size ))
-        goto done;
-
-    /* Strip \Registry\Machine\ prefix and retrieve Wine specific data set by the display driver */
-    lstrcpyW( key_nameW, bufferW + 18 );
-    size = sizeof( state_flags );
-    if (RegGetValueW( HKEY_CURRENT_CONFIG, key_nameW, L"StateFlags", RRF_RT_REG_DWORD, NULL,
-                      &state_flags, &size ))
-        goto done;
-
-    if (!(state_flags & DISPLAY_DEVICE_ATTACHED_TO_DESKTOP))
-        goto done;
-
-    size = sizeof( bufferW );
-    if (RegGetValueW( HKEY_CURRENT_CONFIG, key_nameW, L"GPUID", RRF_RT_REG_SZ, NULL, bufferW, &size ))
-        goto done;
-
-    devinfo = SetupDiCreateDeviceInfoList( &GUID_DEVCLASS_DISPLAY, NULL );
-    device_data.cbSize = sizeof( device_data );
-    SetupDiOpenDeviceInfoW( devinfo, bufferW, NULL, 0, &device_data );
-    if (!SetupDiGetDevicePropertyW( devinfo, &device_data, &DEVPROPKEY_GPU_LUID, &type,
-                                    (BYTE *)&luid, sizeof( luid ), NULL, 0))
-        goto done;
+    if (!(adapter = heap_alloc( sizeof( *adapter ) ))) return STATUS_NO_MEMORY;
 
     EnterCriticalSection( &driver_section );
-    /* D3DKMT_HANDLE is UINT, so we can't use pointer as handle */
-    adapter->handle = ++handle_start;
+    desc->hAdapter = adapter->handle = ++handle_start;
     list_add_tail( &d3dkmt_adapters, &adapter->entry );
     LeaveCriticalSection( &driver_section );
-
-    desc->hAdapter = handle_start;
-    desc->AdapterLuid = luid;
-    desc->VidPnSourceId = index;
-    status = STATUS_SUCCESS;
-
-done:
-    SetupDiDestroyDeviceInfoList( devinfo );
-    release_display_device_init_mutex( mutex );
-    if (status != STATUS_SUCCESS)
-        heap_free( adapter );
-    return status;
+    return STATUS_SUCCESS;
 }
 
 /******************************************************************************
- *		D3DKMTCreateDevice [GDI32.@]
+ *           NtGdiDdDDICreateDevice    (win32u.@)
  */
-NTSTATUS WINAPI D3DKMTCreateDevice( D3DKMT_CREATEDEVICE *desc )
+NTSTATUS WINAPI NtGdiDdDDICreateDevice( D3DKMT_CREATEDEVICE *desc )
 {
     static D3DKMT_HANDLE handle_start = 0;
     struct d3dkmt_adapter *adapter;
@@ -1585,9 +1200,9 @@ NTSTATUS WINAPI D3DKMTCreateDevice( D3DKMT_CREATEDEVICE *desc )
 }
 
 /******************************************************************************
- *		D3DKMTDestroyDevice [GDI32.@]
+ *           NtGdiDdDDIDestroyDevice    (win32u.@)
  */
-NTSTATUS WINAPI D3DKMTDestroyDevice( const D3DKMT_DESTROYDEVICE *desc )
+NTSTATUS WINAPI NtGdiDdDDIDestroyDevice( const D3DKMT_DESTROYDEVICE *desc )
 {
     NTSTATUS status = STATUS_INVALID_PARAMETER;
     D3DKMT_SETVIDPNSOURCEOWNER set_owner_desc;
@@ -1605,7 +1220,7 @@ NTSTATUS WINAPI D3DKMTDestroyDevice( const D3DKMT_DESTROYDEVICE *desc )
         {
             memset( &set_owner_desc, 0, sizeof(set_owner_desc) );
             set_owner_desc.hDevice = desc->hDevice;
-            D3DKMTSetVidPnSourceOwner( &set_owner_desc );
+            NtGdiDdDDISetVidPnSourceOwner( &set_owner_desc );
             list_remove( &device->entry );
             heap_free( device );
             status = STATUS_SUCCESS;
@@ -1618,27 +1233,27 @@ NTSTATUS WINAPI D3DKMTDestroyDevice( const D3DKMT_DESTROYDEVICE *desc )
 }
 
 /******************************************************************************
- *		D3DKMTQueryStatistics [GDI32.@]
+ *           NtGdiDdDDIQueryStatistics    (win32u.@)
  */
-NTSTATUS WINAPI D3DKMTQueryStatistics(D3DKMT_QUERYSTATISTICS *stats)
+NTSTATUS WINAPI NtGdiDdDDIQueryStatistics( D3DKMT_QUERYSTATISTICS *stats )
 {
     FIXME("(%p): stub\n", stats);
     return STATUS_SUCCESS;
 }
 
 /******************************************************************************
- *		D3DKMTSetQueuedLimit [GDI32.@]
+ *           NtGdiDdDDISetQueuedLimit    (win32u.@)
  */
-NTSTATUS WINAPI D3DKMTSetQueuedLimit( D3DKMT_SETQUEUEDLIMIT *desc )
+NTSTATUS WINAPI NtGdiDdDDISetQueuedLimit( D3DKMT_SETQUEUEDLIMIT *desc )
 {
     FIXME( "(%p): stub\n", desc );
     return STATUS_NOT_IMPLEMENTED;
 }
 
 /******************************************************************************
- *		D3DKMTSetVidPnSourceOwner [GDI32.@]
+ *           NtGdiDdDDISetVidPnSourceOwner    (win32u.@)
  */
-NTSTATUS WINAPI D3DKMTSetVidPnSourceOwner( const D3DKMT_SETVIDPNSOURCEOWNER *desc )
+NTSTATUS WINAPI NtGdiDdDDISetVidPnSourceOwner( const D3DKMT_SETVIDPNSOURCEOWNER *desc )
 {
     TRACE("(%p)\n", desc);
 
@@ -1656,9 +1271,9 @@ NTSTATUS WINAPI D3DKMTSetVidPnSourceOwner( const D3DKMT_SETVIDPNSOURCEOWNER *des
 }
 
 /******************************************************************************
- *		D3DKMTCheckVidPnExclusiveOwnership [GDI32.@]
+ *           NtGdiDdDDICheckVidPnExclusiveOwnership    (win32u.@)
  */
-NTSTATUS WINAPI D3DKMTCheckVidPnExclusiveOwnership( const D3DKMT_CHECKVIDPNEXCLUSIVEOWNERSHIP *desc )
+NTSTATUS WINAPI NtGdiDdDDICheckVidPnExclusiveOwnership( const D3DKMT_CHECKVIDPNEXCLUSIVEOWNERSHIP *desc )
 {
     TRACE("(%p)\n", desc);
 
