@@ -18,21 +18,28 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#define NONAMELESSUNION
 #include <unistd.h>
 #include <stdarg.h>
+#include <stdlib.h>
+#include "initguid.h"
 #include "hid.h"
+#include "devguid.h"
+#include "devpkey.h"
 #include "ntddmou.h"
 #include "ntddkbd.h"
 #include "ddk/hidtypes.h"
 #include "ddk/wdm.h"
 #include "regstr.h"
+#include "winuser.h"
 #include "wine/debug.h"
 #include "wine/asm.h"
 #include "wine/list.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(hid);
 
+DEFINE_DEVPROPKEY(DEVPROPKEY_HID_HANDLE, 0xbc62e415, 0xf4fe, 0x405c, 0x8e, 0xda, 0x63, 0x6f, 0xb5, 0x9f, 0x08, 0x98, 2);
+DEFINE_GUID(GUID_DEVINTERFACE_WINEXINPUT, 0x6c53d5fd, 0x6480, 0x440f, 0xb6, 0x18, 0x47, 0x67, 0x50, 0xc5, 0xe1, 0xa6);
+
 #if defined(__i386__) && !defined(_WIN32)
 
 extern void * WINAPI wrap_fastcall_func1( void *func, const void *a );
@@ -84,7 +91,7 @@ static NTSTATUS get_device_id(DEVICE_OBJECT *device, BUS_QUERY_ID_TYPE type, WCH
 
     wcscpy(id, (WCHAR *)irp_status.Information);
     ExFreePool((WCHAR *)irp_status.Information);
-    return irp_status.u.Status;
+    return irp_status.Status;
 }
 
 /* user32 reserves 1 & 2 for winemouse and winekeyboard,
@@ -98,10 +105,36 @@ static UINT32 alloc_rawinput_handle(void)
     return InterlockedIncrement(&counter);
 }
 
+/* make sure bRawData can hold UsagePage and Usage without requiring additional allocation */
+C_ASSERT(offsetof(RAWINPUT, data.hid.bRawData[2 * sizeof(USAGE)]) < sizeof(RAWINPUT));
+
+static void send_wm_input_device_change(BASE_DEVICE_EXTENSION *ext, LPARAM param)
+{
+    HIDP_COLLECTION_DESC *desc = ext->u.pdo.device_desc.CollectionDesc;
+    RAWINPUT rawinput;
+    INPUT input;
+
+    rawinput.header.dwType = RIM_TYPEHID;
+    rawinput.header.dwSize = offsetof(RAWINPUT, data.hid.bRawData[2 * sizeof(USAGE)]);
+    rawinput.header.hDevice = ULongToHandle(ext->u.pdo.rawinput_handle);
+    rawinput.header.wParam = param;
+    rawinput.data.hid.dwCount = 0;
+    rawinput.data.hid.dwSizeHid = 0;
+    ((USAGE *)rawinput.data.hid.bRawData)[0] = desc->UsagePage;
+    ((USAGE *)rawinput.data.hid.bRawData)[1] = desc->Usage;
+
+    input.type = INPUT_HARDWARE;
+    input.hi.uMsg = WM_INPUT_DEVICE_CHANGE;
+    input.hi.wParamH = 0;
+    input.hi.wParamL = 0;
+    __wine_send_input(0, &input, &rawinput);
+}
+
 static NTSTATUS WINAPI driver_add_device(DRIVER_OBJECT *driver, DEVICE_OBJECT *bus_pdo)
 {
     WCHAR device_id[MAX_DEVICE_ID_LEN], instance_id[MAX_DEVICE_ID_LEN];
     BASE_DEVICE_EXTENSION *ext;
+    BOOL is_xinput_class;
     DEVICE_OBJECT *fdo;
     NTSTATUS status;
     minidriver *minidriver;
@@ -135,6 +168,10 @@ static NTSTATUS WINAPI driver_add_device(DRIVER_OBJECT *driver, DEVICE_OBJECT *b
     swprintf(ext->device_id, ARRAY_SIZE(ext->device_id), L"HID\\%s", wcsrchr(device_id, '\\') + 1);
     wcscpy(ext->instance_id, instance_id);
 
+    is_xinput_class = !wcsncmp(device_id, L"WINEXINPUT\\", 7) && wcsstr(device_id, L"&XI_") != NULL;
+    if (is_xinput_class) ext->class_guid = &GUID_DEVINTERFACE_WINEXINPUT;
+    else ext->class_guid = &GUID_DEVINTERFACE_HID;
+
     status = minidriver->AddDevice(minidriver->minidriver.DriverObject, fdo);
     if (status != STATUS_SUCCESS)
     {
@@ -153,19 +190,21 @@ static void create_child(minidriver *minidriver, DEVICE_OBJECT *fdo)
 {
     BASE_DEVICE_EXTENSION *fdo_ext = fdo->DeviceExtension, *pdo_ext;
     HID_DEVICE_ATTRIBUTES attr = {0};
+    HIDP_COLLECTION_DESC *desc;
     HID_DESCRIPTOR descriptor;
     DEVICE_OBJECT *child_pdo;
     BYTE *reportDescriptor;
     UNICODE_STRING string;
     WCHAR pdo_name[255];
+    IO_STATUS_BLOCK io;
     USAGE page, usage;
     NTSTATUS status;
     INT i;
 
-    status = call_minidriver(IOCTL_HID_GET_DEVICE_ATTRIBUTES, fdo, NULL, 0, &attr, sizeof(attr));
-    if (status != STATUS_SUCCESS)
+    call_minidriver( IOCTL_HID_GET_DEVICE_ATTRIBUTES, fdo, NULL, 0, &attr, sizeof(attr), &io );
+    if (io.Status != STATUS_SUCCESS)
     {
-        ERR("Minidriver failed to get Attributes(%x)\n",status);
+        ERR( "Minidriver failed to get attributes, status %#x.\n", io.Status );
         return;
     }
 
@@ -174,25 +213,28 @@ static void create_child(minidriver *minidriver, DEVICE_OBJECT *fdo)
     RtlInitUnicodeString(&string, pdo_name);
     if ((status = IoCreateDevice(fdo->DriverObject, sizeof(*pdo_ext), &string, 0, 0, FALSE, &child_pdo)))
     {
-        ERR("Failed to create child PDO, status %#x.\n", status);
+        ERR( "Failed to create child PDO, status %#x.\n", io.Status );
         return;
     }
     fdo_ext->u.fdo.child_pdo = child_pdo;
 
     pdo_ext = child_pdo->DeviceExtension;
     pdo_ext->u.pdo.parent_fdo = fdo;
+    list_init( &pdo_ext->u.pdo.report_queues );
+    KeInitializeSpinLock( &pdo_ext->u.pdo.report_queues_lock );
     InitializeListHead(&pdo_ext->u.pdo.irp_queue);
     KeInitializeSpinLock(&pdo_ext->u.pdo.irp_queue_lock);
     wcscpy(pdo_ext->device_id, fdo_ext->device_id);
     wcscpy(pdo_ext->instance_id, fdo_ext->instance_id);
+    pdo_ext->class_guid = fdo_ext->class_guid;
 
     pdo_ext->u.pdo.information.VendorID = attr.VendorID;
     pdo_ext->u.pdo.information.ProductID = attr.ProductID;
     pdo_ext->u.pdo.information.VersionNumber = attr.VersionNumber;
     pdo_ext->u.pdo.information.Polled = minidriver->minidriver.DevicesArePolled;
 
-    status = call_minidriver(IOCTL_HID_GET_DEVICE_DESCRIPTOR, fdo, NULL, 0, &descriptor, sizeof(descriptor));
-    if (status != STATUS_SUCCESS)
+    call_minidriver( IOCTL_HID_GET_DEVICE_DESCRIPTOR, fdo, NULL, 0, &descriptor, sizeof(descriptor), &io );
+    if (io.Status != STATUS_SUCCESS)
     {
         ERR("Cannot get Device Descriptor(%x)\n",status);
         IoDeleteDevice(child_pdo);
@@ -209,32 +251,32 @@ static void create_child(minidriver *minidriver, DEVICE_OBJECT *fdo)
         return;
     }
 
-    reportDescriptor = HeapAlloc(GetProcessHeap(), 0, descriptor.DescriptorList[i].wReportLength);
-    status = call_minidriver(IOCTL_HID_GET_REPORT_DESCRIPTOR, fdo, NULL, 0,
-        reportDescriptor, descriptor.DescriptorList[i].wReportLength);
-    if (status != STATUS_SUCCESS)
+    reportDescriptor = malloc(descriptor.DescriptorList[i].wReportLength);
+    call_minidriver( IOCTL_HID_GET_REPORT_DESCRIPTOR, fdo, NULL, 0, reportDescriptor,
+                     descriptor.DescriptorList[i].wReportLength, &io );
+    if (io.Status != STATUS_SUCCESS)
     {
         ERR("Cannot get Report Descriptor(%x)\n",status);
-        HeapFree(GetProcessHeap(), 0, reportDescriptor);
+        free(reportDescriptor);
         IoDeleteDevice(child_pdo);
         return;
     }
 
-    pdo_ext->u.pdo.preparsed_data = ParseDescriptor(reportDescriptor, descriptor.DescriptorList[i].wReportLength);
-    HeapFree(GetProcessHeap(), 0, reportDescriptor);
-    if (!pdo_ext->u.pdo.preparsed_data)
+    io.Status = HidP_GetCollectionDescription( reportDescriptor, descriptor.DescriptorList[i].wReportLength,
+                                               PagedPool, &pdo_ext->u.pdo.device_desc );
+    free(reportDescriptor);
+    if (io.Status != HIDP_STATUS_SUCCESS)
     {
         ERR("Cannot parse Report Descriptor\n");
         IoDeleteDevice(child_pdo);
         return;
     }
 
-    pdo_ext->u.pdo.information.DescriptorSize = pdo_ext->u.pdo.preparsed_data->dwSize;
+    desc = pdo_ext->u.pdo.device_desc.CollectionDesc;
+    pdo_ext->u.pdo.information.DescriptorSize = desc->PreparsedDataLength;
 
-    IoInvalidateDeviceRelations(fdo_ext->u.fdo.hid_ext.PhysicalDeviceObject, BusRelations);
-
-    page = pdo_ext->u.pdo.preparsed_data->caps.UsagePage;
-    usage = pdo_ext->u.pdo.preparsed_data->caps.Usage;
+    page = desc->UsagePage;
+    usage = desc->Usage;
     if (page == HID_USAGE_PAGE_GENERIC && usage == HID_USAGE_GENERIC_MOUSE)
         pdo_ext->u.pdo.rawinput_handle = WINE_MOUSE_HANDLE;
     else if (page == HID_USAGE_PAGE_GENERIC && usage == HID_USAGE_GENERIC_KEYBOARD)
@@ -242,12 +284,22 @@ static void create_child(minidriver *minidriver, DEVICE_OBJECT *fdo)
     else
         pdo_ext->u.pdo.rawinput_handle = alloc_rawinput_handle();
 
+    IoInvalidateDeviceRelations(fdo_ext->u.fdo.hid_ext.PhysicalDeviceObject, BusRelations);
+
+    if ((status = IoSetDevicePropertyData(child_pdo, &DEVPROPKEY_HID_HANDLE, LOCALE_NEUTRAL,
+                                          PLUGPLAY_PROPERTY_PERSISTENT, DEVPROP_TYPE_UINT32,
+                                          sizeof(pdo_ext->u.pdo.rawinput_handle), &pdo_ext->u.pdo.rawinput_handle)))
+    {
+        ERR("Failed to set device handle property, status %x\n", status);
+        IoDeleteDevice(child_pdo);
+        return;
+    }
+
     pdo_ext->u.pdo.poll_interval = DEFAULT_POLL_INTERVAL;
 
-    pdo_ext->u.pdo.ring_buffer = RingBuffer_Create(
-            sizeof(HID_XFER_PACKET) + pdo_ext->u.pdo.preparsed_data->caps.InputReportByteLength);
-
     HID_StartDeviceThread(child_pdo);
+
+    send_wm_input_device_change(pdo_ext, GIDC_ARRIVAL);
 }
 
 static NTSTATUS fdo_pnp(DEVICE_OBJECT *device, IRP *irp)
@@ -270,7 +322,7 @@ static NTSTATUS fdo_pnp(DEVICE_OBJECT *device, IRP *irp)
 
             if (!(devices = ExAllocatePool(PagedPool, offsetof(DEVICE_RELATIONS, Objects[1]))))
             {
-                irp->IoStatus.u.Status = STATUS_NO_MEMORY;
+                irp->IoStatus.Status = STATUS_NO_MEMORY;
                 IoCompleteRequest(irp, IO_NO_INCREMENT);
                 return STATUS_NO_MEMORY;
             }
@@ -287,7 +339,7 @@ static NTSTATUS fdo_pnp(DEVICE_OBJECT *device, IRP *irp)
             }
 
             irp->IoStatus.Information = (ULONG_PTR)devices;
-            irp->IoStatus.u.Status = STATUS_SUCCESS;
+            irp->IoStatus.Status = STATUS_SUCCESS;
             IoSkipCurrentIrpStackLocation(irp);
             return IoCallDriver(ext->u.fdo.hid_ext.NextDeviceObject, irp);
         }
@@ -318,11 +370,24 @@ static NTSTATUS fdo_pnp(DEVICE_OBJECT *device, IRP *irp)
     }
 }
 
+static void remove_pending_irps(BASE_DEVICE_EXTENSION *ext)
+{
+    IRP *irp;
+
+    while ((irp = pop_irp_from_queue(ext)))
+    {
+        irp->IoStatus.Status = STATUS_DELETE_PENDING;
+        IoCompleteRequest(irp, IO_NO_INCREMENT);
+    }
+}
+
 static NTSTATUS pdo_pnp(DEVICE_OBJECT *device, IRP *irp)
 {
     IO_STACK_LOCATION *irpsp = IoGetCurrentIrpStackLocation(irp);
     BASE_DEVICE_EXTENSION *ext = device->DeviceExtension;
-    NTSTATUS status = irp->IoStatus.u.Status;
+    HIDP_COLLECTION_DESC *desc = ext->u.pdo.device_desc.CollectionDesc;
+    NTSTATUS status = irp->IoStatus.Status;
+    KIRQL irql;
 
     TRACE("irp %p, minor function %#x.\n", irp, irpsp->MinorFunction);
 
@@ -387,21 +452,19 @@ static NTSTATUS pdo_pnp(DEVICE_OBJECT *device, IRP *irp)
         }
 
         case IRP_MN_START_DEVICE:
-            if ((status = IoRegisterDeviceInterface(device, &GUID_DEVINTERFACE_HID, NULL, &ext->u.pdo.link_name)))
+            if ((status = IoRegisterDeviceInterface(device, ext->class_guid, NULL, &ext->u.pdo.link_name)))
             {
                 ERR("Failed to register interface, status %#x.\n", status);
                 break;
             }
 
             /* FIXME: This should probably be done in mouhid.sys. */
-            if (ext->u.pdo.preparsed_data->caps.UsagePage == HID_USAGE_PAGE_GENERIC
-                    && ext->u.pdo.preparsed_data->caps.Usage == HID_USAGE_GENERIC_MOUSE)
+            if (desc->UsagePage == HID_USAGE_PAGE_GENERIC && desc->Usage == HID_USAGE_GENERIC_MOUSE)
             {
                 if (!IoRegisterDeviceInterface(device, &GUID_DEVINTERFACE_MOUSE, NULL, &ext->u.pdo.mouse_link_name))
                     ext->u.pdo.is_mouse = TRUE;
             }
-            if (ext->u.pdo.preparsed_data->caps.UsagePage == HID_USAGE_PAGE_GENERIC
-                    && ext->u.pdo.preparsed_data->caps.Usage == HID_USAGE_GENERIC_KEYBOARD)
+            if (desc->UsagePage == HID_USAGE_PAGE_GENERIC && desc->Usage == HID_USAGE_GENERIC_KEYBOARD)
             {
                 if (!IoRegisterDeviceInterface(device, &GUID_DEVINTERFACE_KEYBOARD, NULL, &ext->u.pdo.keyboard_link_name))
                     ext->u.pdo.is_keyboard = TRUE;
@@ -412,18 +475,21 @@ static NTSTATUS pdo_pnp(DEVICE_OBJECT *device, IRP *irp)
                 IoSetDeviceInterfaceState(&ext->u.pdo.mouse_link_name, TRUE);
             if (ext->u.pdo.is_keyboard)
                 IoSetDeviceInterfaceState(&ext->u.pdo.keyboard_link_name, TRUE);
+
+            ext->u.pdo.removed = FALSE;
             status = STATUS_SUCCESS;
             break;
 
         case IRP_MN_REMOVE_DEVICE:
-        {
-            IRP *queued_irp;
+            remove_pending_irps(ext);
+
+            send_wm_input_device_change(ext, GIDC_REMOVAL);
 
             IoSetDeviceInterfaceState(&ext->u.pdo.link_name, FALSE);
             if (ext->u.pdo.is_mouse)
                 IoSetDeviceInterfaceState(&ext->u.pdo.mouse_link_name, FALSE);
             if (ext->u.pdo.is_keyboard)
-                IoSetDeviceInterfaceState(&ext->u.pdo.keyboard_link_name, TRUE);
+                IoSetDeviceInterfaceState(&ext->u.pdo.keyboard_link_name, FALSE);
 
             if (ext->u.pdo.thread)
             {
@@ -432,25 +498,23 @@ static NTSTATUS pdo_pnp(DEVICE_OBJECT *device, IRP *irp)
             }
             CloseHandle(ext->u.pdo.halt_event);
 
-            HeapFree(GetProcessHeap(), 0, ext->u.pdo.preparsed_data);
-            if (ext->u.pdo.ring_buffer)
-                RingBuffer_Destroy(ext->u.pdo.ring_buffer);
-
-            while ((queued_irp = pop_irp_from_queue(ext)))
-            {
-                queued_irp->IoStatus.u.Status = STATUS_DEVICE_REMOVED;
-                IoCompleteRequest(queued_irp, IO_NO_INCREMENT);
-            }
+            HidP_FreeCollectionDescription(&ext->u.pdo.device_desc);
 
             RtlFreeUnicodeString(&ext->u.pdo.link_name);
 
-            irp->IoStatus.u.Status = STATUS_SUCCESS;
+            irp->IoStatus.Status = STATUS_SUCCESS;
             IoCompleteRequest(irp, IO_NO_INCREMENT);
             IoDeleteDevice(device);
             return STATUS_SUCCESS;
-        }
 
         case IRP_MN_SURPRISE_REMOVAL:
+            KeAcquireSpinLock(&ext->u.pdo.lock, &irql);
+            ext->u.pdo.removed = TRUE;
+            KeReleaseSpinLock(&ext->u.pdo.lock, irql);
+
+            remove_pending_irps(ext);
+
+            SetEvent(ext->u.pdo.halt_event);
             status = STATUS_SUCCESS;
             break;
 
@@ -458,7 +522,7 @@ static NTSTATUS pdo_pnp(DEVICE_OBJECT *device, IRP *irp)
             FIXME("Unhandled minor function %#x.\n", irpsp->MinorFunction);
     }
 
-    irp->IoStatus.u.Status = status;
+    irp->IoStatus.Status = status;
     IoCompleteRequest( irp, IO_NO_INCREMENT );
     return status;
 }
@@ -479,7 +543,7 @@ static NTSTATUS WINAPI driver_create(DEVICE_OBJECT *device, IRP *irp)
 
     if (ext->is_fdo)
     {
-        irp->IoStatus.u.Status = STATUS_UNSUCCESSFUL;
+        irp->IoStatus.Status = STATUS_UNSUCCESSFUL;
         IoCompleteRequest(irp, IO_NO_INCREMENT);
         return STATUS_UNSUCCESSFUL;
     }
@@ -518,7 +582,7 @@ static void WINAPI driver_unload(DRIVER_OBJECT *driver)
         if (md->DriverUnload)
             md->DriverUnload(md->minidriver.DriverObject);
         list_remove(&md->entry);
-        HeapFree(GetProcessHeap(), 0, md);
+        free(md);
     }
 }
 
@@ -526,7 +590,7 @@ NTSTATUS WINAPI HidRegisterMinidriver(HID_MINIDRIVER_REGISTRATION *registration)
 {
     minidriver *driver;
 
-    if (!(driver = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*driver))))
+    if (!(driver = calloc(1, sizeof(*driver))))
         return STATUS_NO_MEMORY;
 
     driver->DriverUnload = registration->DriverObject->DriverUnload;
@@ -550,19 +614,16 @@ NTSTATUS WINAPI HidRegisterMinidriver(HID_MINIDRIVER_REGISTRATION *registration)
     return STATUS_SUCCESS;
 }
 
-NTSTATUS call_minidriver(ULONG code, DEVICE_OBJECT *device, void *in_buff, ULONG in_size, void *out_buff, ULONG out_size)
+void call_minidriver( ULONG code, DEVICE_OBJECT *device, void *in_buff, ULONG in_size,
+                      void *out_buff, ULONG out_size, IO_STATUS_BLOCK *io )
 {
     IRP *irp;
-    IO_STATUS_BLOCK io;
     KEVENT event;
 
     KeInitializeEvent(&event, NotificationEvent, FALSE);
 
-    irp = IoBuildDeviceIoControlRequest(code, device, in_buff, in_size,
-        out_buff, out_size, TRUE, &event, &io);
+    irp = IoBuildDeviceIoControlRequest( code, device, in_buff, in_size, out_buff, out_size, TRUE, &event, io );
 
     if (IoCallDriver(device, irp) == STATUS_PENDING)
         KeWaitForSingleObject(&event, Executive, KernelMode, FALSE, NULL);
-
-    return io.u.Status;
 }
