@@ -58,7 +58,9 @@
 #define WIN32_NO_STATUS
 #include "windef.h"
 #include "winternl.h"
+#include "winioctl.h"
 #include "unix_private.h"
+#include "wine/condrv.h"
 #include "wine/exception.h"
 #include "wine/server.h"
 #include "wine/debug.h"
@@ -70,25 +72,8 @@ static ULONG execute_flags = MEM_EXECUTE_OPTION_DISABLE | (sizeof(void *) > size
                                                            MEM_EXECUTE_OPTION_DISABLE_THUNK_EMULATION |
                                                            MEM_EXECUTE_OPTION_PERMANENT : 0);
 
-static const char * const cpu_names[] = { "x86", "x86_64", "ARM", "ARM64" };
-
 static UINT process_error_mode;
 
-static client_cpu_t get_machine_cpu( pe_image_info_t *pe_info )
-{
-    switch (pe_info->machine)
-    {
-    case IMAGE_FILE_MACHINE_I386:
-        if ((is_win64 || is_wow64) && (pe_info->image_flags & IMAGE_FLAGS_ComPlusNativeReady))
-            return CPU_x86_64;
-        return CPU_x86;
-    case IMAGE_FILE_MACHINE_AMD64: return CPU_x86_64;
-    case IMAGE_FILE_MACHINE_ARMNT: return CPU_ARM;
-    case IMAGE_FILE_MACHINE_ARM64: return CPU_ARM64;
-    default: return 0;
-    }
-}
-
 static char **build_argv( const UNICODE_STRING *cmdline, int reserved )
 {
     char **argv, *arg, *src, *dst;
@@ -159,7 +144,7 @@ static char **build_argv( const UNICODE_STRING *cmdline, int reserved )
 /***********************************************************************
  *           get_so_file_info
  */
-static BOOL get_so_file_info( HANDLE handle, pe_image_info_t *info )
+static BOOL get_so_file_info( int fd, pe_image_info_t *info )
 {
     union
     {
@@ -200,12 +185,9 @@ static BOOL get_so_file_info( HANDLE handle, pe_image_info_t *info )
         IMAGE_DOS_HEADER mz;
     } header;
 
-    IO_STATUS_BLOCK io;
-    LARGE_INTEGER offset;
+    off_t pos;
 
-    offset.QuadPart = 0;
-    if (NtReadFile( handle, 0, NULL, NULL, &io, &header, sizeof(header), &offset, 0 )) return FALSE;
-    if (io.Information != sizeof(header)) return FALSE;
+    if (pread( fd, &header, sizeof(header), 0 ) != sizeof(header)) return FALSE;
 
     if (!memcmp( header.elf.magic, "\177ELF", 4 ))
     {
@@ -228,20 +210,19 @@ static BOOL get_so_file_info( HANDLE handle, pe_image_info_t *info )
         if (header.elf.type != 3 /* ET_DYN */) return FALSE;
         if (header.elf.class == 2 /* ELFCLASS64 */)
         {
-            offset.QuadPart = header.elf64.phoff;
+            pos = header.elf64.phoff;
             phnum = header.elf64.phnum;
         }
         else
         {
-            offset.QuadPart = header.elf.phoff;
+            pos = header.elf.phoff;
             phnum = header.elf.phnum;
         }
         while (phnum--)
         {
-            if (NtReadFile( handle, 0, NULL, NULL, &io, &type, sizeof(type), &offset, 0 )) return FALSE;
-            if (io.Information < sizeof(type)) return FALSE;
+            if (pread( fd, &type, sizeof(type), pos ) != sizeof(type)) return FALSE;
             if (type == 3 /* PT_INTERP */) return FALSE;
-            offset.QuadPart += (header.elf.class == 2) ? 56 : 32;
+            pos += (header.elf.class == 2) ? 56 : 32;
         }
         return TRUE;
     }
@@ -305,7 +286,13 @@ static NTSTATUS get_pe_file_info( OBJECT_ATTRIBUTES *attr, HANDLE *handle, pe_im
     }
     else if (status == STATUS_INVALID_IMAGE_NOT_MZ)
     {
-        if (get_so_file_info( *handle, info )) return STATUS_SUCCESS;
+        int unix_fd, needs_close;
+
+        if (!server_get_unix_fd( *handle, FILE_READ_DATA, &unix_fd, &needs_close, NULL, NULL ))
+        {
+            if (get_so_file_info( unix_fd, info )) status = STATUS_SUCCESS;
+            if (needs_close) close( unix_fd );
+        }
     }
     return status;
 }
@@ -368,24 +355,31 @@ static WCHAR *get_nt_pathname( const UNICODE_STRING *str )
  */
 static int get_unix_curdir( const RTL_USER_PROCESS_PARAMETERS *params )
 {
-    UNICODE_STRING nt_name;
+    UNICODE_STRING nt_name, redir;
     OBJECT_ATTRIBUTES attr;
-    IO_STATUS_BLOCK io;
     NTSTATUS status;
     HANDLE handle;
     int fd = -1;
+    char *unix_name;
 
     if (!(nt_name.Buffer = get_nt_pathname( &params->CurrentDirectory.DosPath ))) return -1;
     nt_name.Length = wcslen( nt_name.Buffer ) * sizeof(WCHAR);
 
     InitializeObjectAttributes( &attr, &nt_name, OBJ_CASE_INSENSITIVE, 0, NULL );
-    status = NtOpenFile( &handle, FILE_TRAVERSE | SYNCHRONIZE, &attr, &io,
-                         FILE_SHARE_READ | FILE_SHARE_WRITE,
-                         FILE_DIRECTORY_FILE | FILE_SYNCHRONOUS_IO_NONALERT );
-    free( nt_name.Buffer );
-    if (status) return -1;
+    get_redirect( &attr, &redir );
+    status = nt_to_unix_file_name( &attr, &unix_name, FILE_OPEN );
+    if (status) goto done;
+    status = open_unix_file( &handle, unix_name, FILE_TRAVERSE | SYNCHRONIZE, &attr, 0,
+                             FILE_SHARE_READ | FILE_SHARE_DELETE,
+                             FILE_OPEN, FILE_SYNCHRONOUS_IO_NONALERT, NULL, 0 );
+    free( unix_name );
+    if (status) goto done;
     wine_server_handle_to_fd( handle, FILE_TRAVERSE, &fd, NULL );
     NtClose( handle );
+
+done:
+    free( nt_name.Buffer );
+    free( redir.Buffer );
     return fd;
 }
 
@@ -404,12 +398,23 @@ static void set_stdio_fd( int stdin_fd, int stdout_fd )
         if (stdout_fd == -1) stdout_fd = fd;
     }
 
-    dup2( stdin_fd, 0 );
-    dup2( stdout_fd, 1 );
+    if (stdin_fd != 0) dup2( stdin_fd, 0 );
+    if (stdout_fd != 1) dup2( stdout_fd, 1 );
     if (fd != -1) close( fd );
 }
 
 
+/***********************************************************************
+ *           is_unix_console_handle
+ */
+static BOOL is_unix_console_handle( HANDLE handle )
+{
+    IO_STATUS_BLOCK io = {0};
+    return !NtDeviceIoControlFile( handle, NULL, NULL, NULL, &io, IOCTL_CONDRV_IS_UNIX,
+                                   NULL, 0, NULL, 0 );
+}
+
+
 /***********************************************************************
  *           spawn_process
  */
@@ -421,8 +426,13 @@ static NTSTATUS spawn_process( const RTL_USER_PROCESS_PARAMETERS *params, int so
     pid_t pid;
     char **argv;
 
-    wine_server_handle_to_fd( params->hStdInput, FILE_READ_DATA, &stdin_fd, NULL );
-    wine_server_handle_to_fd( params->hStdOutput, FILE_WRITE_DATA, &stdout_fd, NULL );
+    if (wine_server_handle_to_fd( params->hStdInput, FILE_READ_DATA, &stdin_fd, NULL ) &&
+        isatty(0) && is_unix_console_handle( params->hStdInput ))
+        stdin_fd = 0;
+
+    if (wine_server_handle_to_fd( params->hStdOutput, FILE_WRITE_DATA, &stdout_fd, NULL ) &&
+        isatty(1) && is_unix_console_handle( params->hStdOutput ))
+        stdout_fd = 1;
 
     if (!(pid = fork()))  /* child */
     {
@@ -437,8 +447,8 @@ static NTSTATUS spawn_process( const RTL_USER_PROCESS_PARAMETERS *params, int so
             }
             else set_stdio_fd( stdin_fd, stdout_fd );
 
-            if (stdin_fd != -1) close( stdin_fd );
-            if (stdout_fd != -1) close( stdout_fd );
+            if (stdin_fd != -1 && stdin_fd != 0) close( stdin_fd );
+            if (stdout_fd != -1 && stdout_fd != 1) close( stdout_fd );
 
             if (winedebug) putenv( winedebug );
             if (unixdir != -1)
@@ -465,8 +475,8 @@ static NTSTATUS spawn_process( const RTL_USER_PROCESS_PARAMETERS *params, int so
     }
     else status = STATUS_NO_MEMORY;
 
-    if (stdin_fd != -1) close( stdin_fd );
-    if (stdout_fd != -1) close( stdout_fd );
+    if (stdin_fd != -1 && stdin_fd != 0) close( stdin_fd );
+    if (stdout_fd != -1 && stdout_fd != 1) close( stdout_fd );
     return status;
 }
 
@@ -501,8 +511,13 @@ static NTSTATUS fork_and_exec( OBJECT_ATTRIBUTES *attr, int unixdir,
         fcntl( fd[1], F_SETFD, FD_CLOEXEC );
     }
 
-    wine_server_handle_to_fd( params->hStdInput, FILE_READ_DATA, &stdin_fd, NULL );
-    wine_server_handle_to_fd( params->hStdOutput, FILE_WRITE_DATA, &stdout_fd, NULL );
+    if (wine_server_handle_to_fd( params->hStdInput, FILE_READ_DATA, &stdin_fd, NULL ) &&
+        isatty(0) && is_unix_console_handle( params->hStdInput ))
+        stdin_fd = 0;
+
+    if (wine_server_handle_to_fd( params->hStdOutput, FILE_WRITE_DATA, &stdout_fd, NULL ) &&
+        isatty(1) && is_unix_console_handle( params->hStdOutput ))
+        stdout_fd = 1;
 
     if (!(pid = fork()))  /* child */
     {
@@ -511,7 +526,7 @@ static NTSTATUS fork_and_exec( OBJECT_ATTRIBUTES *attr, int unixdir,
             close( fd[0] );
 
             if (params->ConsoleFlags ||
-                params->ConsoleHandle == (HANDLE)1 /* KERNEL32_CONSOLE_ALLOC */ ||
+                params->ConsoleHandle == CONSOLE_HANDLE_ALLOC ||
                 (params->hStdInput == INVALID_HANDLE_VALUE && params->hStdOutput == INVALID_HANDLE_VALUE))
             {
                 setsid();
@@ -519,8 +534,8 @@ static NTSTATUS fork_and_exec( OBJECT_ATTRIBUTES *attr, int unixdir,
             }
             else set_stdio_fd( stdin_fd, stdout_fd );
 
-            if (stdin_fd != -1) close( stdin_fd );
-            if (stdout_fd != -1) close( stdout_fd );
+            if (stdin_fd != -1 && stdin_fd != 0) close( stdin_fd );
+            if (stdout_fd != -1 && stdout_fd != 1) close( stdout_fd );
 
             /* Reset signals that we previously set to SIG_IGN */
             signal( SIGPIPE, SIG_DFL );
@@ -567,8 +582,8 @@ static NTSTATUS fork_and_exec( OBJECT_ATTRIBUTES *attr, int unixdir,
     else status = STATUS_NO_MEMORY;
 
     close( fd[0] );
-    if (stdin_fd != -1) close( stdin_fd );
-    if (stdout_fd != -1) close( stdout_fd );
+    if (stdin_fd != -1 && stdin_fd != 0) close( stdin_fd );
+    if (stdout_fd != -1 && stdout_fd != 1) close( stdout_fd );
 done:
     free( unix_name );
     return status;
@@ -622,9 +637,9 @@ NTSTATUS WINAPI NtCreateUserProcess( HANDLE *process_handle_ptr, HANDLE *thread_
     UNICODE_STRING redir, path = {0};
     OBJECT_ATTRIBUTES attr, empty_attr = { sizeof(empty_attr) };
     SIZE_T i, attr_count = (ps_attr->TotalLength - sizeof(ps_attr->TotalLength)) / sizeof(PS_ATTRIBUTE);
-    const PS_ATTRIBUTE *handles_attr = NULL;
-    data_size_t handles_size;
-    obj_handle_t *handles;
+    const PS_ATTRIBUTE *handles_attr = NULL, *jobs_attr = NULL;
+    data_size_t handles_size, jobs_size;
+    obj_handle_t *handles, *jobs;
 
     for (i = 0; i < attr_count; i++)
     {
@@ -647,6 +662,9 @@ NTSTATUS WINAPI NtCreateUserProcess( HANDLE *process_handle_ptr, HANDLE *thread_
             if (process_flags & PROCESS_CREATE_FLAGS_INHERIT_HANDLES)
                 handles_attr = &ps_attr->Attributes[i];
             break;
+        case PS_ATTRIBUTE_JOB_LIST:
+            jobs_attr = &ps_attr->Attributes[i];
+            break;
         default:
             if (ps_attr->Attributes[i].Attribute & PS_ATTRIBUTE_INPUT)
                 FIXME( "unhandled input attribute %lx\n", ps_attr->Attributes[i].Attribute );
@@ -684,6 +702,13 @@ NTSTATUS WINAPI NtCreateUserProcess( HANDLE *process_handle_ptr, HANDLE *thread_
         goto done;
     }
 
+    if ((status = alloc_handle_list( jobs_attr, &jobs, &jobs_size )))
+    {
+        free( objattr );
+        free( handles );
+        goto done;
+    }
+
     /* create the socket for the new process */
 
     if (socketpair( PF_UNIX, SOCK_STREAM, 0, socketfd ) == -1)
@@ -691,6 +716,7 @@ NTSTATUS WINAPI NtCreateUserProcess( HANDLE *process_handle_ptr, HANDLE *thread_
         status = STATUS_TOO_MANY_OPENED_FILES;
         free( objattr );
         free( handles );
+        free( jobs );
         goto done;
     }
 #ifdef SO_PASSCRED
@@ -714,11 +740,12 @@ NTSTATUS WINAPI NtCreateUserProcess( HANDLE *process_handle_ptr, HANDLE *thread_
         req->flags          = process_flags;
         req->socket_fd      = socketfd[1];
         req->access         = process_access;
-        req->cpu            = get_machine_cpu( &pe_info );
         req->info_size      = startup_info_size;
         req->handles_size   = handles_size;
+        req->jobs_size      = jobs_size;
         wine_server_add_data( req, objattr, attr_len );
         wine_server_add_data( req, handles, handles_size );
+        wine_server_add_data( req, jobs, jobs_size );
         wine_server_add_data( req, startup_info, startup_info_size );
         wine_server_add_data( req, params->Environment, env_size );
         if (!(status = wine_server_call( req )))
@@ -731,6 +758,7 @@ NTSTATUS WINAPI NtCreateUserProcess( HANDLE *process_handle_ptr, HANDLE *thread_
     SERVER_END_REQ;
     free( objattr );
     free( handles );
+    free( jobs );
 
     if (status)
     {
@@ -740,8 +768,8 @@ NTSTATUS WINAPI NtCreateUserProcess( HANDLE *process_handle_ptr, HANDLE *thread_
             ERR( "64-bit application %s not supported in 32-bit prefix\n", debugstr_us(&path) );
             break;
         case STATUS_INVALID_IMAGE_FORMAT:
-            ERR( "%s not supported on this installation (%s binary)\n",
-                 debugstr_us(&path), cpu_names[get_machine_cpu(&pe_info)] );
+            ERR( "%s not supported on this installation (machine %04x)\n",
+                 debugstr_us(&path), pe_info.machine );
             break;
         }
         goto done;
@@ -875,10 +903,11 @@ void fill_vm_counters( VM_COUNTERS_EX *pvmi, int unix_pid )
 {
 #if defined(MACH_TASK_BASIC_INFO)
     struct mach_task_basic_info info;
+    mach_msg_type_number_t infoCount;
 
     if (unix_pid != -1) return; /* FIXME: Retrieve information for other processes. */
 
-    mach_msg_type_number_t infoCount = MACH_TASK_BASIC_INFO_COUNT;
+    infoCount = MACH_TASK_BASIC_INFO_COUNT;
     if(task_info(mach_task_self(), MACH_TASK_BASIC_INFO, (task_info_t)&info, &infoCount) == KERN_SUCCESS)
     {
         pvmi->VirtualSize = info.resident_size + info.virtual_size;
@@ -967,7 +996,6 @@ NTSTATUS WINAPI NtQueryInformationProcess( HANDLE handle, PROCESSINFOCLASS class
     UNIMPLEMENTED_INFO_CLASS(ProcessWx86Information);
     UNIMPLEMENTED_INFO_CLASS(ProcessPriorityBoost);
     UNIMPLEMENTED_INFO_CLASS(ProcessDeviceMap);
-    UNIMPLEMENTED_INFO_CLASS(ProcessSessionInformation);
     UNIMPLEMENTED_INFO_CLASS(ProcessForegroundInformation);
     UNIMPLEMENTED_INFO_CLASS(ProcessLUIDDeviceMapsEnabled);
     UNIMPLEMENTED_INFO_CLASS(ProcessBreakOnTermination);
@@ -994,6 +1022,15 @@ NTSTATUS WINAPI NtQueryInformationProcess( HANDLE handle, PROCESSINFOCLASS class
                             pbi.BasePriority = reply->priority;
                             pbi.UniqueProcessId = reply->pid;
                             pbi.InheritedFromUniqueProcessId = reply->ppid;
+#ifndef _WIN64
+                            if (is_wow64)
+                            {
+                                if (reply->machine != native_machine)
+                                    pbi.PebBaseAddress = (PEB *)((char *)pbi.PebBaseAddress + 0x1000);
+                                else
+                                    pbi.PebBaseAddress = NULL;
+                            }
+#endif
                         }
                     }
                     SERVER_END_REQ;
@@ -1243,27 +1280,37 @@ NTSTATUS WINAPI NtQueryInformationProcess( HANDLE handle, PROCESSINFOCLASS class
         else ret = STATUS_INFO_LENGTH_MISMATCH;
         break;
 
+    case ProcessSessionInformation:
+        len = sizeof(DWORD);
+        if (size == len)
+        {
+            SERVER_START_REQ(get_process_info)
+            {
+                req->handle = wine_server_obj_handle( handle );
+                if (!(ret = wine_server_call( req )))
+                    *(DWORD *)info = reply->session_id;
+            }
+            SERVER_END_REQ;
+        }
+        else ret = STATUS_INFO_LENGTH_MISMATCH;
+        break;
+
     case ProcessWow64Information:
         len = sizeof(ULONG_PTR);
         if (size != len) ret = STATUS_INFO_LENGTH_MISMATCH;
         else if (!info) ret = STATUS_ACCESS_VIOLATION;
         else if (!handle) ret = STATUS_INVALID_HANDLE;
+        else if (handle == GetCurrentProcess()) *(ULONG_PTR *)info = !!NtCurrentTeb()->WowTebOffset;
         else
         {
             ULONG_PTR val = 0;
 
-            if (handle == GetCurrentProcess()) val = is_wow64;
-            else if (is_win64 || is_wow64)
+            SERVER_START_REQ( get_process_info )
             {
-                SERVER_START_REQ( get_process_info )
-                {
-                    req->handle = wine_server_obj_handle( handle );
-                    if (!(ret = wine_server_call( req )))
-                        val = (reply->machine != IMAGE_FILE_MACHINE_AMD64 &&
-                               reply->machine != IMAGE_FILE_MACHINE_ARM64);
-                }
-                SERVER_END_REQ;
+                req->handle = wine_server_obj_handle( handle );
+                if (!(ret = wine_server_call( req ))) val = (reply->machine != native_machine);
             }
+            SERVER_END_REQ;
             *(ULONG_PTR *)info = val;
         }
         break;
@@ -1440,6 +1487,17 @@ NTSTATUS WINAPI NtSetInformationProcess( HANDLE handle, PROCESSINFOCLASS class,
         }
         break;
 
+    case ProcessInstrumentationCallback:
+    {
+        PROCESS_INSTRUMENTATION_CALLBACK_INFORMATION *instr = info;
+
+        FIXME( "ProcessInstrumentationCallback stub.\n" );
+
+        if (size < sizeof(*instr)) return STATUS_INFO_LENGTH_MISMATCH;
+        ret = STATUS_SUCCESS;
+        break;
+    }
+
     case ProcessThreadStackAllocation:
     {
         void *addr = NULL;
@@ -1462,6 +1520,17 @@ NTSTATUS WINAPI NtSetInformationProcess( HANDLE handle, PROCESSINFOCLASS class,
         break;
     }
 
+    case ProcessWineMakeProcessSystem:
+        if (size != sizeof(HANDLE *)) return STATUS_INFO_LENGTH_MISMATCH;
+        SERVER_START_REQ( make_process_system )
+        {
+            req->handle = wine_server_obj_handle( handle );
+            if (!(ret = wine_server_call( req )))
+                *(HANDLE *)info = wine_server_ptr_handle( reply->event );
+        }
+        SERVER_END_REQ;
+        return ret;
+
     default:
         FIXME( "(%p,0x%08x,%p,0x%08x) stub\n", handle, class, info, size );
         ret = STATUS_NOT_IMPLEMENTED;
@@ -1479,6 +1548,8 @@ NTSTATUS WINAPI NtOpenProcess( HANDLE *handle, ACCESS_MASK access,
 {
     NTSTATUS status;
 
+    *handle = 0;
+
     SERVER_START_REQ( open_process )
     {
         req->pid        = HandleToULong( id->UniqueProcess );
@@ -1582,22 +1653,3 @@ NTSTATUS WINAPI NtDebugContinue( HANDLE handle, CLIENT_ID *client, NTSTATUS stat
     SERVER_END_REQ;
     return ret;
 }
-
-
-/***********************************************************************
- *           __wine_make_process_system   (NTDLL.@)
- *
- * Mark the current process as a system process.
- * Returns the event that is signaled when all non-system processes have exited.
- */
-HANDLE CDECL __wine_make_process_system(void)
-{
-    HANDLE ret = 0;
-
-    SERVER_START_REQ( make_process_system )
-    {
-        if (!wine_server_call( req )) ret = wine_server_ptr_handle( reply->event );
-    }
-    SERVER_END_REQ;
-    return ret;
-}
