@@ -680,7 +680,7 @@ static HRESULT Stream_LoadLocation( IStream *stm,
  *
  *    0          Length of the block (4 bytes, usually 0x314)
  *    4          tag (dword)
- *    8          string data in ASCII
+ *    8          string data in ANSI
  *    8+0x104    string data in UNICODE
  *
  * In the original Win32 implementation the buffers are not initialized
@@ -2158,7 +2158,10 @@ static HRESULT WINAPI IShellLinkW_fnSetPath(IShellLinkW * iface, LPCWSTR pszFile
         if (*pszFile == '\0')
             *buffer = '\0';
         else if (!GetFullPathNameW(pszFile, MAX_PATH, buffer, &fname))
-	    return E_FAIL;
+        {
+            heap_free(unquoted);
+            return E_FAIL;
+        }
         else if(!PathFileExistsW(buffer) &&
 		!SearchPathW(NULL, pszFile, NULL, MAX_PATH, buffer, NULL))
 	  hr = S_FALSE;
