@@ -57,7 +57,6 @@ struct irp_call
 };
 
 static void irp_call_dump( struct object *obj, int verbose );
-static int irp_call_signaled( struct object *obj, struct wait_queue_entry *entry );
 static void irp_call_destroy( struct object *obj );
 
 static const struct object_ops irp_call_ops =
@@ -65,10 +64,10 @@ static const struct object_ops irp_call_ops =
     sizeof(struct irp_call),          /* size */
     &no_type,                         /* type */
     irp_call_dump,                    /* dump */
-    add_queue,                        /* add_queue */
-    remove_queue,                     /* remove_queue */
-    irp_call_signaled,                /* signaled */
-    no_satisfied,                     /* satisfied */
+    no_add_queue,                     /* add_queue */
+    NULL,                             /* remove_queue */
+    NULL,                             /* signaled */
+    NULL,                             /* satisfied */
     no_signal,                        /* signal */
     no_get_fd,                        /* get_fd */
     default_map_access,               /* map_access */
@@ -202,11 +201,12 @@ static struct list *device_file_get_kernel_obj_list( struct object *obj );
 static int device_file_close_handle( struct object *obj, struct process *process, obj_handle_t handle );
 static void device_file_destroy( struct object *obj );
 static enum server_fd_type device_file_get_fd_type( struct fd *fd );
-static int device_file_read( struct fd *fd, struct async *async, file_pos_t pos );
-static int device_file_write( struct fd *fd, struct async *async, file_pos_t pos );
-static int device_file_flush( struct fd *fd, struct async *async );
-static int device_file_ioctl( struct fd *fd, ioctl_code_t code, struct async *async );
-static void device_file_reselect_async( struct fd *fd, struct async_queue *queue );
+static void device_file_read( struct fd *fd, struct async *async, file_pos_t pos );
+static void device_file_write( struct fd *fd, struct async *async, file_pos_t pos );
+static void device_file_flush( struct fd *fd, struct async *async );
+static void device_file_ioctl( struct fd *fd, ioctl_code_t code, struct async *async );
+static void device_file_cancel_async( struct fd *fd, struct async *async );
+static void device_file_get_volume_info( struct fd *fd, struct async *async, unsigned int info_class );
 
 static const struct object_ops device_file_ops =
 {
@@ -241,10 +241,11 @@ static const struct fd_ops device_file_fd_ops =
     device_file_write,                /* write */
     device_file_flush,                /* flush */
     default_fd_get_file_info,         /* get_file_info */
-    no_fd_get_volume_info,            /* get_volume_info */
+    device_file_get_volume_info,      /* get_volume_info */
     device_file_ioctl,                /* ioctl */
+    device_file_cancel_async,         /* cancel_async */
     default_fd_queue_async,           /* queue_async */
-    device_file_reselect_async        /* reselect_async */
+    default_fd_reselect_async,        /* reselect_async */
 };
 
 
@@ -334,13 +335,6 @@ static void irp_call_dump( struct object *obj, int verbose )
     fprintf( stderr, "IRP call file=%p\n", irp->file );
 }
 
-static int irp_call_signaled( struct object *obj, struct wait_queue_entry *entry )
-{
-    struct irp_call *irp = (struct irp_call *)obj;
-
-    return !irp->file;  /* file is cleared once the irp has completed */
-}
-
 static void irp_call_destroy( struct object *obj )
 {
     struct irp_call *irp = (struct irp_call *)obj;
@@ -376,11 +370,6 @@ static struct irp_call *create_irp( struct device_file *file, const irp_params_t
         irp->user_ptr = 0;
 
         if (async) irp->iosb = async_get_iosb( async );
-        if (!irp->iosb && !(irp->iosb = create_iosb( NULL, 0, 0 )))
-        {
-            release_object( irp );
-            irp = NULL;
-        }
     }
     return irp;
 }
@@ -389,28 +378,19 @@ static void set_irp_result( struct irp_call *irp, unsigned int status,
                             const void *out_data, data_size_t out_size, data_size_t result )
 {
     struct device_file *file = irp->file;
-    struct iosb *iosb = irp->iosb;
 
     if (!file) return;  /* already finished */
 
-    /* FIXME: handle the STATUS_PENDING case */
-    iosb->status = status;
-    iosb->result = result;
-    iosb->out_size = min( iosb->out_size, out_size );
-    if (iosb->out_size && !(iosb->out_data = memdup( out_data, iosb->out_size )))
-        iosb->out_size = 0;
-
     /* remove it from the device queue */
     list_remove( &irp->dev_entry );
     irp->file = NULL;
     if (irp->async)
     {
-        if (result) status = STATUS_ALERTED;
-        async_terminate( irp->async, status );
+        out_size = min( irp->iosb->out_size, out_size );
+        async_request_complete_alloc( irp->async, status, result, out_size, out_data );
         release_object( irp->async );
         irp->async = NULL;
     }
-    wake_up( &irp->obj, 0 );
 
     release_object( irp );  /* no longer on the device queue */
     release_object( file );
@@ -594,6 +574,10 @@ static int fill_irp_params( struct device_manager *manager, struct irp_call *irp
         irp->params.ioctl.file     = get_kernel_object_ptr( manager, &irp->file->obj );
         irp->params.ioctl.out_size = irp->iosb->out_size;
         break;
+    case IRP_CALL_VOLUME:
+        irp->params.volume.file     = get_kernel_object_ptr( manager, &irp->file->obj );
+        irp->params.volume.out_size = irp->iosb->out_size;
+        break;
     }
 
     *params = irp->params;
@@ -613,17 +597,16 @@ static void free_irp_params( struct irp_call *irp )
 }
 
 /* queue an irp to the device */
-static int queue_irp( struct device_file *file, const irp_params_t *params, struct async *async )
+static void queue_irp( struct device_file *file, const irp_params_t *params, struct async *async )
 {
     struct irp_call *irp = create_irp( file, params, async );
-    if (!irp) return 0;
+    if (!irp) return;
 
     fd_queue_async( file->fd, async, ASYNC_TYPE_WAIT );
     irp->async = (struct async *)grab_object( async );
     add_irp_to_queue( file->device->manager, irp, current );
     release_object( irp );
-    set_error( STATUS_PENDING );
-    return 0;
+    async_set_unknown_status( async );
 }
 
 static enum server_fd_type device_file_get_fd_type( struct fd *fd )
@@ -631,7 +614,18 @@ static enum server_fd_type device_file_get_fd_type( struct fd *fd )
     return FD_TYPE_DEVICE;
 }
 
-static int device_file_read( struct fd *fd, struct async *async, file_pos_t pos )
+static void device_file_get_volume_info( struct fd *fd, struct async *async, unsigned int info_class )
+{
+    struct device_file *file = get_fd_user( fd );
+    irp_params_t params;
+
+    memset( &params, 0, sizeof(params) );
+    params.volume.type = IRP_CALL_VOLUME;
+    params.volume.info_class = info_class;
+    queue_irp( file, &params, async );
+}
+
+static void device_file_read( struct fd *fd, struct async *async, file_pos_t pos )
 {
     struct device_file *file = get_fd_user( fd );
     irp_params_t params;
@@ -640,10 +634,10 @@ static int device_file_read( struct fd *fd, struct async *async, file_pos_t pos
     params.read.type = IRP_CALL_READ;
     params.read.key  = 0;
     params.read.pos  = pos;
-    return queue_irp( file, &params, async );
+    queue_irp( file, &params, async );
 }
 
-static int device_file_write( struct fd *fd, struct async *async, file_pos_t pos )
+static void device_file_write( struct fd *fd, struct async *async, file_pos_t pos )
 {
     struct device_file *file = get_fd_user( fd );
     irp_params_t params;
@@ -652,20 +646,20 @@ static int device_file_write( struct fd *fd, struct async *async, file_pos_t pos
     params.write.type = IRP_CALL_WRITE;
     params.write.key  = 0;
     params.write.pos  = pos;
-    return queue_irp( file, &params, async );
+    queue_irp( file, &params, async );
 }
 
-static int device_file_flush( struct fd *fd, struct async *async )
+static void device_file_flush( struct fd *fd, struct async *async )
 {
     struct device_file *file = get_fd_user( fd );
     irp_params_t params;
 
     memset( &params, 0, sizeof(params) );
     params.flush.type = IRP_CALL_FLUSH;
-    return queue_irp( file, &params, async );
+    queue_irp( file, &params, async );
 }
 
-static int device_file_ioctl( struct fd *fd, ioctl_code_t code, struct async *async )
+static void device_file_ioctl( struct fd *fd, ioctl_code_t code, struct async *async )
 {
     struct device_file *file = get_fd_user( fd );
     irp_params_t params;
@@ -673,7 +667,7 @@ static int device_file_ioctl( struct fd *fd, ioctl_code_t code, struct async *as
     memset( &params, 0, sizeof(params) );
     params.ioctl.type = IRP_CALL_IOCTL;
     params.ioctl.code = code;
-    return queue_irp( file, &params, async );
+    queue_irp( file, &params, async );
 }
 
 static void cancel_irp_call( struct irp_call *irp )
@@ -693,21 +687,21 @@ static void cancel_irp_call( struct irp_call *irp )
         add_irp_to_queue( irp->file->device->manager, cancel_irp, NULL );
         release_object( cancel_irp );
     }
-
-    set_irp_result( irp, STATUS_CANCELLED, NULL, 0, 0 );
 }
 
-static void device_file_reselect_async( struct fd *fd, struct async_queue *queue )
+static void device_file_cancel_async( struct fd *fd, struct async *async )
 {
     struct device_file *file = get_fd_user( fd );
     struct irp_call *irp;
 
     LIST_FOR_EACH_ENTRY( irp, &file->requests, struct irp_call, dev_entry )
-        if (irp->iosb->status != STATUS_PENDING)
+    {
+        if (irp->async == async)
         {
             cancel_irp_call( irp );
             return;
         }
+    }
 }
 
 static struct device *create_device( struct object *root, const struct unicode_str *name,
@@ -967,7 +961,7 @@ DECL_HANDLER(get_next_device_request)
              * so we need to do it now */
             cancel_irp_call( irp );
         else if (irp->async)
-            set_async_pending( irp->async, irp->file && is_fd_overlapped( irp->file->fd ) );
+            set_async_pending( irp->async );
 
         free_irp_params( irp );
         release_object( irp );
@@ -987,15 +981,21 @@ DECL_HANDLER(get_next_device_request)
         reply->client_tid    = get_thread_id( thread );
 
         iosb = irp->iosb;
-        reply->in_size = iosb->in_size;
-        if (iosb->in_size > get_reply_max_size()) set_error( STATUS_BUFFER_OVERFLOW );
+        if (iosb)
+            reply->in_size = iosb->in_size;
+
+        if (iosb && iosb->in_size > get_reply_max_size())
+            set_error( STATUS_BUFFER_OVERFLOW );
         else if (!irp->file || (reply->next = alloc_handle( current->process, irp, 0, 0 )))
         {
             if (fill_irp_params( manager, irp, &reply->params ))
             {
-                set_reply_data_ptr( iosb->in_data, iosb->in_size );
-                iosb->in_data = NULL;
-                iosb->in_size = 0;
+                if (iosb)
+                {
+                    set_reply_data_ptr( iosb->in_data, iosb->in_size );
+                    iosb->in_data = NULL;
+                    iosb->in_size = 0;
+                }
                 list_remove( &irp->mgr_entry );
                 list_init( &irp->mgr_entry );
                 /* we already own the object if it's only on manager queue */
@@ -1018,12 +1018,9 @@ DECL_HANDLER(set_irp_result)
 
     if ((irp = (struct irp_call *)get_handle_obj( current->process, req->handle, 0, &irp_call_ops )))
     {
-        if (!irp->canceled)
-            set_irp_result( irp, req->status, get_req_data(), get_req_data_size(), req->size );
-        else if(irp->user_ptr) /* cancel already queued */
-            set_error( STATUS_MORE_PROCESSING_REQUIRED );
-        else /* we may be still dispatching the IRP. don't bother queuing cancel if it's already complete */
-            irp->canceled = 0;
+        set_irp_result( irp, req->status, get_req_data(), get_req_data_size(), req->size );
+        /* we may be still dispatching the IRP. don't bother queuing cancel if it's already complete */
+        irp->canceled = 0;
         close_handle( current->process, req->handle );  /* avoid an extra round-trip for close */
         release_object( irp );
     }
