@@ -143,10 +143,10 @@ static struct security_descriptor *pipe_end_get_sd( struct object *obj );
 static int pipe_end_set_sd( struct object *obj, const struct security_descriptor *sd,
                             unsigned int set_info );
 static WCHAR *pipe_end_get_full_name( struct object *obj, data_size_t *len );
-static int pipe_end_read( struct fd *fd, struct async *async, file_pos_t pos );
-static int pipe_end_write( struct fd *fd, struct async *async_data, file_pos_t pos );
-static int pipe_end_flush( struct fd *fd, struct async *async );
-static int pipe_end_get_volume_info( struct fd *fd, struct async *async, unsigned int info_class );
+static void pipe_end_read( struct fd *fd, struct async *async, file_pos_t pos );
+static void pipe_end_write( struct fd *fd, struct async *async_data, file_pos_t pos );
+static void pipe_end_flush( struct fd *fd, struct async *async );
+static void pipe_end_get_volume_info( struct fd *fd, struct async *async, unsigned int info_class );
 static void pipe_end_reselect_async( struct fd *fd, struct async_queue *queue );
 static void pipe_end_get_file_info( struct fd *fd, obj_handle_t handle, unsigned int info_class );
 
@@ -157,7 +157,7 @@ static struct object *pipe_server_lookup_name( struct object *obj, struct unicod
 static struct object *pipe_server_open_file( struct object *obj, unsigned int access,
                                              unsigned int sharing, unsigned int options );
 static void pipe_server_destroy( struct object *obj);
-static int pipe_server_ioctl( struct fd *fd, ioctl_code_t code, struct async *async );
+static void pipe_server_ioctl( struct fd *fd, ioctl_code_t code, struct async *async );
 
 static const struct object_ops pipe_server_ops =
 {
@@ -179,7 +179,7 @@ static const struct object_ops pipe_server_ops =
     NULL,                         /* unlink_name */
     pipe_server_open_file,        /* open_file */
     no_kernel_obj_list,           /* get_kernel_obj_list */
-    fd_close_handle,              /* close_handle */
+    no_close_handle,              /* close_handle */
     pipe_server_destroy           /* destroy */
 };
 
@@ -194,13 +194,14 @@ static const struct fd_ops pipe_server_fd_ops =
     pipe_end_get_file_info,       /* get_file_info */
     pipe_end_get_volume_info,     /* get_volume_info */
     pipe_server_ioctl,            /* ioctl */
+    default_fd_cancel_async,      /* cancel_async */
     no_fd_queue_async,            /* queue_async */
     pipe_end_reselect_async       /* reselect_async */
 };
 
 /* client end functions */
 static void pipe_client_dump( struct object *obj, int verbose );
-static int pipe_client_ioctl( struct fd *fd, ioctl_code_t code, struct async *async );
+static void pipe_client_ioctl( struct fd *fd, ioctl_code_t code, struct async *async );
 
 static const struct object_ops pipe_client_ops =
 {
@@ -222,7 +223,7 @@ static const struct object_ops pipe_client_ops =
     NULL,                         /* unlink_name */
     no_open_file,                 /* open_file */
     no_kernel_obj_list,           /* get_kernel_obj_list */
-    fd_close_handle,              /* close_handle */
+    no_close_handle,              /* close_handle */
     pipe_end_destroy              /* destroy */
 };
 
@@ -237,6 +238,7 @@ static const struct fd_ops pipe_client_fd_ops =
     pipe_end_get_file_info,       /* get_file_info */
     pipe_end_get_volume_info,     /* get_volume_info */
     pipe_client_ioctl,            /* ioctl */
+    default_fd_cancel_async,      /* cancel_async */
     no_fd_queue_async,            /* queue_async */
     pipe_end_reselect_async       /* reselect_async */
 };
@@ -275,7 +277,7 @@ static const struct object_ops named_pipe_device_ops =
 static void named_pipe_device_file_dump( struct object *obj, int verbose );
 static struct fd *named_pipe_device_file_get_fd( struct object *obj );
 static WCHAR *named_pipe_device_file_get_full_name( struct object *obj, data_size_t *len );
-static int named_pipe_device_ioctl( struct fd *fd, ioctl_code_t code, struct async *async );
+static void named_pipe_device_ioctl( struct fd *fd, ioctl_code_t code, struct async *async );
 static enum server_fd_type named_pipe_device_file_get_fd_type( struct fd *fd );
 static void named_pipe_device_file_destroy( struct object *obj );
 
@@ -299,7 +301,7 @@ static const struct object_ops named_pipe_device_file_ops =
     NULL,                                    /* unlink_name */
     no_open_file,                            /* open_file */
     no_kernel_obj_list,                      /* get_kernel_obj_list */
-    fd_close_handle,                         /* close_handle */
+    no_close_handle,                         /* close_handle */
     named_pipe_device_file_destroy           /* destroy */
 };
 
@@ -314,6 +316,7 @@ static const struct fd_ops named_pipe_device_fd_ops =
     default_fd_get_file_info,                /* get_file_info */
     no_fd_get_volume_info,                   /* get_volume_info */
     named_pipe_device_ioctl,                 /* ioctl */
+    default_fd_cancel_async,                 /* cancel_async */
     default_fd_queue_async,                  /* queue_async */
     default_fd_reselect_async                /* reselect_async */
 };
@@ -390,9 +393,7 @@ static void wake_message( struct pipe_message *message, data_size_t result )
     message->async = NULL;
     if (!async) return;
 
-    message->iosb->status = STATUS_SUCCESS;
-    message->iosb->result = result;
-    async_terminate( async, message->iosb->result ? STATUS_ALERTED : STATUS_SUCCESS );
+    async_request_complete( async, STATUS_SUCCESS, result, 0, NULL );
     release_object( async );
 }
 
@@ -580,14 +581,14 @@ static void named_pipe_device_file_destroy( struct object *obj )
     release_object( file->device );
 }
 
-static int pipe_end_flush( struct fd *fd, struct async *async )
+static void pipe_end_flush( struct fd *fd, struct async *async )
 {
     struct pipe_end *pipe_end = get_fd_user( fd );
 
     if (!pipe_end->pipe)
     {
         set_error( STATUS_PIPE_DISCONNECTED );
-        return 0;
+        return;
     }
 
     if (pipe_end->connection && !list_empty( &pipe_end->connection->message_queue ))
@@ -595,7 +596,6 @@ static int pipe_end_flush( struct fd *fd, struct async *async )
         fd_queue_async( pipe_end->fd, async, ASYNC_TYPE_WAIT );
         set_error( STATUS_PENDING );
     }
-    return 1;
 }
 
 static void pipe_end_get_file_info( struct fd *fd, obj_handle_t handle, unsigned int info_class )
@@ -742,7 +742,7 @@ static WCHAR *pipe_end_get_full_name( struct object *obj, data_size_t *len )
     return pipe_end->pipe->obj.ops->get_full_name( &pipe_end->pipe->obj, len );
 }
 
-static int pipe_end_get_volume_info( struct fd *fd, struct async *async, unsigned int info_class )
+static void pipe_end_get_volume_info( struct fd *fd, struct async *async, unsigned int info_class )
 {
     switch (info_class)
     {
@@ -762,19 +762,22 @@ static int pipe_end_get_volume_info( struct fd *fd, struct async *async, unsigne
     default:
         set_error( STATUS_NOT_IMPLEMENTED );
     }
-    return 0;
 }
 
-static void message_queue_read( struct pipe_end *pipe_end, struct iosb *iosb )
+static void message_queue_read( struct pipe_end *pipe_end, struct async *async )
 {
+    struct iosb *iosb = async_get_iosb( async );
+    unsigned int status = STATUS_SUCCESS;
     struct pipe_message *message;
+    data_size_t out_size;
 
     if (pipe_end->flags & NAMED_PIPE_MESSAGE_STREAM_READ)
     {
         message = LIST_ENTRY( list_head(&pipe_end->message_queue), struct pipe_message, entry );
-        iosb->out_size = min( iosb->out_size, message->iosb->in_size - message->read_pos );
-        iosb->status = message->read_pos + iosb->out_size < message->iosb->in_size
-            ? STATUS_BUFFER_OVERFLOW : STATUS_SUCCESS;
+        out_size = min( iosb->out_size, message->iosb->in_size - message->read_pos );
+
+        if (message->read_pos + out_size < message->iosb->in_size)
+            status = STATUS_BUFFER_OVERFLOW;
     }
     else
     {
@@ -784,14 +787,13 @@ static void message_queue_read( struct pipe_end *pipe_end, struct iosb *iosb )
             avail += message->iosb->in_size - message->read_pos;
             if (avail >= iosb->out_size) break;
         }
-        iosb->out_size = min( iosb->out_size, avail );
-        iosb->status = STATUS_SUCCESS;
+        out_size = min( iosb->out_size, avail );
     }
 
     message = LIST_ENTRY( list_head(&pipe_end->message_queue), struct pipe_message, entry );
     if (!message->read_pos && message->iosb->in_size == iosb->out_size) /* fast path */
     {
-        iosb->out_data = message->iosb->in_data;
+        async_request_complete( async, status, out_size, out_size, message->iosb->in_data );
         message->iosb->in_data = NULL;
         wake_message( message, message->iosb->in_size );
         free_message( message );
@@ -801,17 +803,17 @@ static void message_queue_read( struct pipe_end *pipe_end, struct iosb *iosb )
         data_size_t write_pos = 0, writing;
         char *buf = NULL;
 
-        if (iosb->out_size && !(buf = iosb->out_data = malloc( iosb->out_size )))
+        if (out_size && !(buf = malloc( out_size )))
         {
-            iosb->out_size = 0;
-            iosb->status = STATUS_NO_MEMORY;
+            async_terminate( async, STATUS_NO_MEMORY );
+            release_object( iosb );
             return;
         }
 
         do
         {
             message = LIST_ENTRY( list_head(&pipe_end->message_queue), struct pipe_message, entry );
-            writing = min( iosb->out_size - write_pos, message->iosb->in_size - message->read_pos );
+            writing = min( out_size - write_pos, message->iosb->in_size - message->read_pos );
             if (writing) memcpy( buf + write_pos, (const char *)message->iosb->in_data + message->read_pos, writing );
             write_pos += writing;
             message->read_pos += writing;
@@ -820,9 +822,12 @@ static void message_queue_read( struct pipe_end *pipe_end, struct iosb *iosb )
                 wake_message(message, message->iosb->in_size);
                 free_message(message);
             }
-        } while (write_pos < iosb->out_size);
+        } while (write_pos < out_size);
+
+        async_request_complete( async, status, out_size, out_size, buf );
     }
-    iosb->result = iosb->out_size;
+
+    release_object( iosb );
 }
 
 /* We call async_terminate in our reselect implementation, which causes recursive reselect.
@@ -834,16 +839,12 @@ static void reselect_write_queue( struct pipe_end *pipe_end );
 static void reselect_read_queue( struct pipe_end *pipe_end, int reselect_write )
 {
     struct async *async;
-    struct iosb *iosb;
 
     ignore_reselect = 1;
     while (!list_empty( &pipe_end->message_queue ) && (async = find_pending_async( &pipe_end->read_q )))
     {
-        iosb = async_get_iosb( async );
-        message_queue_read( pipe_end, iosb );
-        async_terminate( async, iosb->result ? STATUS_ALERTED : iosb->status );
+        message_queue_read( pipe_end, async );
         release_object( async );
-        release_object( iosb );
         reselect_write = 1;
     }
     ignore_reselect = 0;
@@ -894,7 +895,7 @@ static void reselect_write_queue( struct pipe_end *pipe_end )
     reselect_read_queue( reader, 0 );
 }
 
-static int pipe_end_read( struct fd *fd, struct async *async, file_pos_t pos )
+static void pipe_end_read( struct fd *fd, struct async *async, file_pos_t pos )
 {
     struct pipe_end *pipe_end = get_fd_user( fd );
 
@@ -904,28 +905,27 @@ static int pipe_end_read( struct fd *fd, struct async *async, file_pos_t pos )
         if ((pipe_end->flags & NAMED_PIPE_NONBLOCKING_MODE) && list_empty( &pipe_end->message_queue ))
         {
             set_error( STATUS_PIPE_EMPTY );
-            return 0;
+            return;
         }
         break;
     case FILE_PIPE_DISCONNECTED_STATE:
         set_error( STATUS_PIPE_DISCONNECTED );
-        return 0;
+        return;
     case FILE_PIPE_LISTENING_STATE:
         set_error( STATUS_PIPE_LISTENING );
-        return 0;
+        return;
     case FILE_PIPE_CLOSING_STATE:
         if (!list_empty( &pipe_end->message_queue )) break;
         set_error( STATUS_PIPE_BROKEN );
-        return 0;
+        return;
     }
 
     queue_async( &pipe_end->read_q, async );
     reselect_read_queue( pipe_end, 0 );
     set_error( STATUS_PENDING );
-    return 1;
 }
 
-static int pipe_end_write( struct fd *fd, struct async *async, file_pos_t pos )
+static void pipe_end_write( struct fd *fd, struct async *async, file_pos_t pos )
 {
     struct pipe_end *pipe_end = get_fd_user( fd );
     struct pipe_message *message;
@@ -937,27 +937,26 @@ static int pipe_end_write( struct fd *fd, struct async *async, file_pos_t pos )
         break;
     case FILE_PIPE_DISCONNECTED_STATE:
         set_error( STATUS_PIPE_DISCONNECTED );
-        return 0;
+        return;
     case FILE_PIPE_LISTENING_STATE:
         set_error( STATUS_PIPE_LISTENING );
-        return 0;
+        return;
     case FILE_PIPE_CLOSING_STATE:
         set_error( STATUS_PIPE_CLOSING );
-        return 0;
+        return;
     }
 
-    if (!pipe_end->pipe->message_mode && !get_req_data_size()) return 1;
+    if (!pipe_end->pipe->message_mode && !get_req_data_size()) return;
 
     iosb = async_get_iosb( async );
     message = queue_message( pipe_end->connection, iosb );
     release_object( iosb );
-    if (!message) return 0;
+    if (!message) return;
 
     message->async = (struct async *)grab_object( async );
     queue_async( &pipe_end->write_q, async );
     reselect_read_queue( pipe_end->connection, 1 );
     set_error( STATUS_PENDING );
-    return 1;
 }
 
 static void pipe_end_reselect_async( struct fd *fd, struct async_queue *queue )
@@ -977,7 +976,7 @@ static enum server_fd_type pipe_end_get_fd_type( struct fd *fd )
     return FD_TYPE_PIPE;
 }
 
-static int pipe_end_peek( struct pipe_end *pipe_end )
+static void pipe_end_peek( struct pipe_end *pipe_end )
 {
     unsigned reply_size = get_reply_max_size();
     FILE_PIPE_PEEK_BUFFER *buffer;
@@ -988,7 +987,7 @@ static int pipe_end_peek( struct pipe_end *pipe_end )
     if (reply_size < offsetof( FILE_PIPE_PEEK_BUFFER, Data ))
     {
         set_error( STATUS_INFO_LENGTH_MISMATCH );
-        return 0;
+        return;
     }
     reply_size -= offsetof( FILE_PIPE_PEEK_BUFFER, Data );
 
@@ -999,10 +998,10 @@ static int pipe_end_peek( struct pipe_end *pipe_end )
     case FILE_PIPE_CLOSING_STATE:
         if (!list_empty( &pipe_end->message_queue )) break;
         set_error( STATUS_PIPE_BROKEN );
-        return 0;
+        return;
     default:
         set_error( pipe_end->pipe ? STATUS_INVALID_PIPE_STATE : STATUS_PIPE_DISCONNECTED );
-        return 0;
+        return;
     }
 
     LIST_FOR_EACH_ENTRY( message, &pipe_end->message_queue, struct pipe_message, entry )
@@ -1016,7 +1015,7 @@ static int pipe_end_peek( struct pipe_end *pipe_end )
         reply_size = min( reply_size, message_length );
     }
 
-    if (!(buffer = set_reply_data_size( offsetof( FILE_PIPE_PEEK_BUFFER, Data[reply_size] )))) return 0;
+    if (!(buffer = set_reply_data_size( offsetof( FILE_PIPE_PEEK_BUFFER, Data[reply_size] )))) return;
     buffer->NamedPipeState    = pipe_end->state;
     buffer->ReadDataAvailable = avail;
     buffer->NumberOfMessages  = 0;  /* FIXME */
@@ -1035,10 +1034,9 @@ static int pipe_end_peek( struct pipe_end *pipe_end )
         }
     }
     if (message_length > reply_size) set_error( STATUS_BUFFER_OVERFLOW );
-    return 1;
 }
 
-static int pipe_end_transceive( struct pipe_end *pipe_end, struct async *async )
+static void pipe_end_transceive( struct pipe_end *pipe_end, struct async *async )
 {
     struct pipe_message *message;
     struct iosb *iosb;
@@ -1046,20 +1044,20 @@ static int pipe_end_transceive( struct pipe_end *pipe_end, struct async *async )
     if (!pipe_end->connection)
     {
         set_error( pipe_end->pipe ? STATUS_INVALID_PIPE_STATE : STATUS_PIPE_DISCONNECTED );
-        return 0;
+        return;
     }
 
     if (!(pipe_end->flags & NAMED_PIPE_MESSAGE_STREAM_READ))
     {
         set_error( STATUS_INVALID_READ_MODE );
-        return 0;
+        return;
     }
 
     /* not allowed if we already have read data buffered */
     if (!list_empty( &pipe_end->message_queue ))
     {
         set_error( STATUS_PIPE_BUSY );
-        return 0;
+        return;
     }
 
     iosb = async_get_iosb( async );
@@ -1068,16 +1066,15 @@ static int pipe_end_transceive( struct pipe_end *pipe_end, struct async *async )
     /* transaction never blocks on write, so just queue a message without async */
     message = queue_message( pipe_end->connection, iosb );
     release_object( iosb );
-    if (!message) return 0;
+    if (!message) return;
     reselect_read_queue( pipe_end->connection, 0 );
 
     queue_async( &pipe_end->read_q, async );
     reselect_read_queue( pipe_end, 0 );
     set_error( STATUS_PENDING );
-    return 1;
 }
 
-static int pipe_end_get_connection_attribute( struct pipe_end *pipe_end )
+static void pipe_end_get_connection_attribute( struct pipe_end *pipe_end )
 {
     const char *attr = get_req_data();
     data_size_t value_size, attr_size = get_req_data_size();
@@ -1096,38 +1093,40 @@ static int pipe_end_get_connection_attribute( struct pipe_end *pipe_end )
     else
     {
         set_error( STATUS_ILLEGAL_FUNCTION );
-        return 0;
+        return;
     }
 
     if (get_reply_max_size() < value_size)
     {
         set_error( STATUS_INFO_LENGTH_MISMATCH );
-        return 0;
+        return;
     }
 
     set_reply_data( value, value_size );
-    return 1;
 }
 
-static int pipe_end_ioctl( struct pipe_end *pipe_end, ioctl_code_t code, struct async *async )
+static void pipe_end_ioctl( struct pipe_end *pipe_end, ioctl_code_t code, struct async *async )
 {
     switch(code)
     {
     case FSCTL_PIPE_GET_CONNECTION_ATTRIBUTE:
-        return pipe_end_get_connection_attribute( pipe_end );
+        pipe_end_get_connection_attribute( pipe_end );
+        break;
 
     case FSCTL_PIPE_PEEK:
-        return pipe_end_peek( pipe_end );
+        pipe_end_peek( pipe_end );
+        break;
 
     case FSCTL_PIPE_TRANSCEIVE:
-        return pipe_end_transceive( pipe_end, async );
+        pipe_end_transceive( pipe_end, async );
+        break;
 
     default:
-        return default_fd_ioctl( pipe_end->fd, code, async );
+        default_fd_ioctl( pipe_end->fd, code, async );
     }
 }
 
-static int pipe_server_ioctl( struct fd *fd, ioctl_code_t code, struct async *async )
+static void pipe_server_ioctl( struct fd *fd, ioctl_code_t code, struct async *async )
 {
     struct pipe_server *server = get_fd_user( fd );
 
@@ -1144,21 +1143,21 @@ static int pipe_server_ioctl( struct fd *fd, ioctl_code_t code, struct async *as
             break;
         case FILE_PIPE_CONNECTED_STATE:
             set_error( STATUS_PIPE_CONNECTED );
-            return 0;
+            return;
         case FILE_PIPE_CLOSING_STATE:
             set_error( STATUS_PIPE_CLOSING );
-            return 0;
+            return;
         }
 
         if (server->pipe_end.flags & NAMED_PIPE_NONBLOCKING_MODE)
         {
             set_error( STATUS_PIPE_LISTENING );
-            return 0;
+            return;
         }
         queue_async( &server->listen_q, async );
         async_wake_up( &server->pipe_end.pipe->waiters, STATUS_SUCCESS );
         set_error( STATUS_PENDING );
-        return 1;
+        return;
 
     case FSCTL_PIPE_DISCONNECT:
         switch(server->pipe_end.state)
@@ -1173,32 +1172,32 @@ static int pipe_server_ioctl( struct fd *fd, ioctl_code_t code, struct async *as
             break;
         case FILE_PIPE_LISTENING_STATE:
             set_error( STATUS_PIPE_LISTENING );
-            return 0;
+            return;
         case FILE_PIPE_DISCONNECTED_STATE:
             set_error( STATUS_PIPE_DISCONNECTED );
-            return 0;
+            return;
         }
 
         pipe_end_disconnect( &server->pipe_end, STATUS_PIPE_DISCONNECTED );
-        return 1;
+        return;
 
     case FSCTL_PIPE_IMPERSONATE:
         if (current->process->token) /* FIXME: use the client token */
         {
             struct token *token;
             if (!(token = token_duplicate( current->process->token, 0, SecurityImpersonation, NULL, NULL, 0, NULL, 0 )))
-                return 0;
+                return;
             if (current->token) release_object( current->token );
             current->token = token;
         }
-        return 1;
+        return;
 
     default:
-        return pipe_end_ioctl( &server->pipe_end, code, async );
+        pipe_end_ioctl( &server->pipe_end, code, async );
     }
 }
 
-static int pipe_client_ioctl( struct fd *fd, ioctl_code_t code, struct async *async )
+static void pipe_client_ioctl( struct fd *fd, ioctl_code_t code, struct async *async )
 {
     struct pipe_end *client = get_fd_user( fd );
 
@@ -1206,10 +1205,10 @@ static int pipe_client_ioctl( struct fd *fd, ioctl_code_t code, struct async *as
     {
     case FSCTL_PIPE_LISTEN:
         set_error( STATUS_ILLEGAL_FUNCTION );
-        return 0;
+        return;
 
     default:
-        return pipe_end_ioctl( client, code, async );
+        pipe_end_ioctl( client, code, async );
     }
 }
 
@@ -1327,7 +1326,7 @@ static struct object *named_pipe_open_file( struct object *obj, unsigned int acc
     return &client->obj;
 }
 
-static int named_pipe_device_ioctl( struct fd *fd, ioctl_code_t code, struct async *async )
+static void named_pipe_device_ioctl( struct fd *fd, ioctl_code_t code, struct async *async )
 {
     struct named_pipe_device *device = get_fd_user( fd );
 
@@ -1345,11 +1344,11 @@ static int named_pipe_device_ioctl( struct fd *fd, ioctl_code_t code, struct asy
                 size < FIELD_OFFSET(FILE_PIPE_WAIT_FOR_BUFFER, Name[buffer->NameLength/sizeof(WCHAR)]))
             {
                 set_error( STATUS_INVALID_PARAMETER );
-                return 0;
+                return;
             }
             name.str = buffer->Name;
             name.len = (buffer->NameLength / sizeof(WCHAR)) * sizeof(WCHAR);
-            if (!(pipe = open_named_object( &device->obj, &named_pipe_ops, &name, 0 ))) return 0;
+            if (!(pipe = open_named_object( &device->obj, &named_pipe_ops, &name, 0 ))) return;
 
             if (list_empty( &pipe->listeners ))
             {
@@ -1360,11 +1359,11 @@ static int named_pipe_device_ioctl( struct fd *fd, ioctl_code_t code, struct asy
             }
 
             release_object( pipe );
-            return 1;
+            return;
         }
 
     default:
-        return default_fd_ioctl( fd, code, async );
+        default_fd_ioctl( fd, code, async );
     }
 }
 
