@@ -1420,411 +1420,14 @@ static int cmp_link_name( const void *e1, const void *e2 )
 }
 
 
-/* output dispatcher for system calls */
-static void output_syscall_dispatcher( int count, const char *variant )
-{
-    const unsigned int invalid_param = 0xc000000d; /* STATUS_INVALID_PARAMETER */
-    const char *symbol = strmake( "__wine_syscall_dispatcher%s", variant );
-    unsigned int i;
-
-    output( "\t.align %d\n", get_alignment(4) );
-    output( "\t%s\n", func_declaration(symbol) );
-    output( "%s\n", asm_globl(symbol) );
-    output_cfi( ".cfi_startproc" );
-    switch (target_cpu)
-    {
-    case CPU_x86:
-        output( "\tpushl %%ebp\n" );
-        output_cfi( ".cfi_adjust_cfa_offset 4\n" );
-        output_cfi( ".cfi_rel_offset %%ebp,0\n" );
-        output( "\tmovl %%esp,%%ebp\n" );
-        output_cfi( ".cfi_def_cfa_register %%ebp\n" );
-        output( "\tleal -0x2c(%%esp),%%esp\n" );
-        output( "\tmovl %%ebx,-0x14(%%ebp)\n" );
-        output_cfi( ".cfi_rel_offset %%ebx,-0x14\n" );
-        output( "\tmovl %%edi,-0x08(%%ebp)\n" );
-        output_cfi( ".cfi_rel_offset %%edi,-0x08\n" );
-        output( "\tmovl %%esi,-0x04(%%ebp)\n" );
-        output_cfi( ".cfi_rel_offset %%esi,-0x04\n" );
-        output( "\tpushfl\n" );
-        output( "\tmovw %%gs,-0x1a(%%ebp)\n" );
-        output( "\tmovw %%fs,-0x1c(%%ebp)\n" );
-        output( "\tmovw %%es,-0x1e(%%ebp)\n" );
-        output( "\tmovw %%ds,-0x20(%%ebp)\n" );
-        output( "\tmovw %%ss,-0x22(%%ebp)\n" );
-        output( "\tmovw %%cs,-0x24(%%ebp)\n" );
-        output( "\tleal 8(%%ebp),%%ecx\n" );
-        output( "\tmovl %%ecx,-0x28(%%ebp)\n" ); /* frame->esp */
-        output( "\tmovl 4(%%ebp),%%ecx\n" );
-        output( "\tmovl %%ecx,-0x2c(%%ebp)\n" ); /* frame->eip */
-        output( "\tsubl $0x2c0,%%esp\n") ;
-        output( "\tandl $~63,%%esp\n" );
-        if (!*variant)
-        {
-            output( "\tfnsave (%%esp)\n" );
-            output( "\tfwait\n" );
-        }
-        else if(!strcmp( variant, "_fxsave" ))
-        {
-            output( "\tfxsave (%%esp)\n" );
-        }
-        else if(!strcmp( variant, "_xsave" ))
-        {
-            output( "\tmovl %%eax,%%ecx\n ");
-            output( "\tmovl $7,%%eax\n" );
-            output( "\txorl %%edx,%%edx\n" );
-            for (i = 0; i < 6; i++)
-                output( "\tmovl %%edx,0x%x(%%esp)\n", 0x200 + i * 4 );
-            output( "\txsave (%%esp)\n" );
-            output( "\tmovl %%ecx,%%eax\n ");
-        }
-        else /* _xsavec */
-        {
-            output( "\tmovl %%eax,%%ecx\n ");
-            output( "\tmovl $7,%%eax\n" );
-            output( "\txorl %%edx,%%edx\n" );
-            for (i = 0; i < 16; i++)
-                output( "\tmovl %%edx,0x%x(%%esp)\n", 0x200 + i * 4 );
-            output( "\txsavec (%%esp)\n" );
-            output( "\tmovl %%ecx,%%eax\n ");
-        }
-        output( "\tleal -0x30(%%ebp),%%ecx\n" );
-        output( "\tmovl %%ecx,%%fs:0x1f8\n" );  /* x86_thread_data()->syscall_frame */
-        output( "\tcmpl $%u,%%eax\n", count );
-        output( "\tjae 4f\n" );
-        if (UsePIC)
-        {
-            output( "\tmovl %%eax,%%edx\n" );
-            output( "\tcall %s\n", asm_name("__wine_spec_get_pc_thunk_eax") );
-            output( "1:\tmovzbl .Lsyscall_args-1b(%%eax,%%edx,1),%%ecx\n" );
-            needs_get_pc_thunk = 1;
-        }
-        else output( "\tmovzbl .Lsyscall_args(%%eax),%%ecx\n" );
-        output( "\tsubl %%ecx,%%esp\n" );
-        output( "\tshrl $2,%%ecx\n" );
-        output( "\tleal 12(%%ebp),%%esi\n" );
-        output( "\tandl $~15,%%esp\n" );
-        output( "\tmovl %%esp,%%edi\n" );
-        output( "\tcld\n" );
-        output( "\trep; movsl\n" );
-        if (UsePIC)
-            output( "\tcall *.Lsyscall_table-1b(%%eax,%%edx,4)\n" );
-        else
-            output( "\tcall *.Lsyscall_table(,%%eax,4)\n" );
-        output( "2:\tmovl $0,%%fs:0x1f8\n" );
-        output( "\tleal -0x2f0(%%ebp),%%ebx\n") ;
-        output( "\tandl $~63,%%ebx\n" );
-        if (!*variant)
-        {
-            output( "\tfrstor (%%ebx)\n" );
-            output( "\tfwait\n" );
-        }
-        else if(!strcmp( variant, "_fxsave" ))
-        {
-            output( "\tfxrstor (%%ebx)\n" );
-        }
-        else
-        {
-            output( "\tmovl %%eax,%%ecx\n" );
-            output( "\tmovl $7,%%eax\n" );
-            output( "\txorl %%edx,%%edx\n" );
-            output( "\txrstor (%%ebx)\n" );
-            output( "\tmovl %%ecx,%%eax\n" );
-        }
-        output( "\tleal -0x30(%%ebp),%%ebx\n" );
-        output_cfi( ".cfi_def_cfa_register %%ebx" );
-        output_cfi( ".cfi_adjust_cfa_offset 0x30\n" );
-        output( "\tmovl %%eax,0x18(%%ebx)\n" );
-        output( "\tmovw 0x16(%%ebx),%%gs\n" );
-        output( "\tmovw 0x14(%%ebx),%%fs\n" );
-        output( "\tmovw 0x12(%%ebx),%%es\n" );
-        output( "\tmovl 0x28(%%ebx),%%edi\n" );
-        output_cfi( ".cfi_same_value %%edi" );
-        output( "\tmovl 0x2c(%%ebx),%%esi\n" );
-        output_cfi( ".cfi_same_value %%esi" );
-        output( "\tmovl (%%ebp),%%ebp\n" );
-        output_cfi( ".cfi_same_value %%ebp" );
-        output( "\tmovw %%ss,%%cx\n" );
-        output( "\tcmpw 0x0e(%%ebx),%%cx\n" );
-        output( "\tjne 3f\n" );
-        /* As soon as we have switched stacks the context structure could
-         * be invalid (when signal handlers are executed for example). Copy
-         * values on the target stack before changing ESP. */
-        output( "\tmovl 0x08(%%ebx),%%ecx\n" );
-        output( "\tleal -3*4(%%ecx),%%ecx\n" );
-        output( "\tmovl (%%ebx),%%edx\n" );
-        output( "\tmovl %%edx,2*4(%%ecx)\n" );
-        output( "\tmovl 0x0c(%%ebx),%%edx\n" );
-        output( "\tmovl %%edx,1*4(%%ecx)\n" );
-        output( "\tmovl 0x04(%%ebx),%%edx\n" );
-        output( "\tmovl %%edx,0*4(%%ecx)\n" );
-        output( "\tpushl 0x10(%%ebx)\n" );
-        output( "\tmovl 0x1c(%%ebx),%%ebx\n" );
-        output_cfi( ".cfi_same_value %%ebx" );
-        output( "\tpopl %%ds\n" );
-        output( "\tmovl %%ecx,%%esp\n" );
-        output( "\tiret\n" );
-        /* Restore the context when the stack segment changes. We can't use
-         * the same code as above because we do not know if the stack segment
-         * is 16 or 32 bit, and 'movl' will throw an exception when we try to
-         * access memory above the limit. */
-        output( "\t3:\tmovl 0x18(%%ebx),%%ecx\n" );
-        output( "\tmovw 0x0e(%%ebx),%%ss\n" );
-        output( "\tmovl 0x08(%%ebx),%%esp\n" );
-        output( "\tpushl 0x00(%%ebx)\n" );
-        output( "\tpushl 0x0c(%%ebx)\n" );
-        output( "\tpushl 0x04(%%ebx)\n" );
-        output( "\tpushl 0x10(%%ebx)\n" );
-        output( "\tmovl 0x1c(%%ebx),%%ebx\n" );
-        output( "\tpopl %%ds\n" );
-        output( "\tiret\n" );
-        output( "4:\tmovl $0x%x,%%eax\n", invalid_param );
-        output( "\tjmp 2b\n" );
-        break;
-    case CPU_x86_64:
-        output( "\tpushq %%rbp\n" );
-        output_cfi( ".cfi_adjust_cfa_offset 8" );
-        output_cfi( ".cfi_rel_offset %%rbp,0" );
-        output( "\tmovq %%rsp,%%rbp\n" );
-        output_cfi( ".cfi_def_cfa_register %%rbp" );
-        output( "\tleaq -0x10(%%rbp),%%rsp\n" );
-        output( "\tpushfq\n" );
-        output( "\tsubq $0x3c0,%%rsp\n" );
-        output( "\tandq $~63,%%rsp\n" );
-        output( "\tmovq %%rbx,-0x90(%%rbp)\n" );
-        output_cfi( ".cfi_rel_offset %%rbx,-144" );
-        output( "\tmovq %%rsi,-0x78(%%rbp)\n" );
-        output_cfi( ".cfi_rel_offset %%rsi,-120" );
-        output( "\tmovq %%rdi,-0x70(%%rbp)\n" );
-        output_cfi( ".cfi_rel_offset %%rdi,-112" );
-        output( "\tmovq %%r12,-0x48(%%rbp)\n" );
-        output_cfi( ".cfi_rel_offset %%r12,-72" );
-        output( "\tmovq %%r13,-0x40(%%rbp)\n" );
-        output( "\tmovq %%r14,-0x38(%%rbp)\n" );
-        output( "\tmovq %%r15,-0x30(%%rbp)\n" );
-        /* Legends of Runeterra hooks the first system call return instruction, and
-         * depends on us returning to it. Adjust the return address accordingly. */
-        output( "\tsubq $0xb,0x8(%%rbp)\n" );
-        output( "\tmovq 0x8(%%rbp),%%rbx\n" );
-        output( "\tmovq %%rbx,-0x28(%%rbp)\n" );
-        output( "\tleaq 0x10(%%rbp),%%rbx\n" );
-        output( "\tmovq %%rbx,-0x10(%%rbp)\n" );
-        output( "\tmovw %%cs,-0x20(%%rbp)\n" );
-        output( "\tmovw %%ds,-0x1e(%%rbp)\n" );
-        output( "\tmovw %%es,-0x1c(%%rbp)\n" );
-        output( "\tmovw %%fs,-0x1a(%%rbp)\n" );
-        output( "\tmovw %%ss,-0x8(%%rbp)\n" );
-        output( "\tmovw %%gs,-0x6(%%rbp)\n" );
-        output( "\tmovq %%rsp,%%r12\n" );
-        output( "\tmovq %%rax,%%r11\n" );
-        if (!*variant)
-        {
-            output( "\tfxsave64 (%%r12)\n" );
-        }
-        else
-        {
-            output( "\tmovl $7,%%eax\n" );
-            output( "\tmovq %%rdx,%%rsi\n" );
-            output( "\txorq %%rdx,%%rdx\n" );
-            output( "\tmovq %%rdx,0x200(%%r12)\n" );
-            output( "\tmovq %%rdx,0x208(%%r12)\n" );
-            output( "\tmovq %%rdx,0x210(%%r12)\n" );
-            if (!strcmp( variant, "_xsavec" ))
-            {
-                output( "\tmovq %%rdx,0x218(%%r12)\n" );
-                output( "\tmovq %%rdx,0x220(%%r12)\n" );
-                output( "\tmovq %%rdx,0x228(%%r12)\n" );
-                output( "\tmovq %%rdx,0x230(%%r12)\n" );
-                output( "\tmovq %%rdx,0x238(%%r12)\n" );
-                output( "\txsavec64 (%%r12)\n" );
-            }
-            else
-                output( "\txsave64 (%%r12)\n" );
-            output( "\tmovq %%rsi,%%rdx\n" );
-        }
-        output( "\tmovq %%gs:0x30,%%rcx\n" );
-        output( "\tleaq -0x98(%%rbp),%%rbx\n" );
-        output( "\tmovq %%rbx,0x328(%%rcx)\n" );  /* amd64_thread_data()->syscall_frame */
-        output( "\tcmpq $%u,%%r11\n", count );
-        output( "\tjae 3f\n" );
-        output( "\tleaq .Lsyscall_args(%%rip),%%rcx\n" );
-        output( "\tmovzbl (%%rcx,%%r11),%%ecx\n" );
-        output( "\tsubq $0x20,%%rcx\n" );
-        output( "\tjbe 1f\n" );
-        output( "\tsubq %%rcx,%%rsp\n" );
-        output( "\tshrq $3,%%rcx\n" );
-        output( "\tleaq 0x38(%%rbp),%%rsi\n" );
-        output( "\tandq $~15,%%rsp\n\t" );
-        output( "\tmovq %%rsp,%%rdi\n" );
-        output( "\tcld\n" );
-        output( "\trep; movsq\n" );
-        output( "1:\tmovq %%r10,%%rcx\n" );
-        output( "\tsubq $0x20,%%rsp\n" );
-        output( "\tleaq .Lsyscall_table(%%rip),%%r10\n" );
-        output( "\tcallq *(%%r10,%%r11,8)\n" );
-        output( "2:\tmovq %%gs:0x30,%%rcx\n" );
-        output( "\tmovq $0,0x328(%%rcx)\n" );
-        if (!*variant)
-        {
-            output( "\tfxrstor64 (%%r12)\n" );
-        }
-        else
-        {
-            output( "\tmovq %%rax,%%r11\n" );
-            output( "\tmovl $7,%%eax\n" );
-            output( "\txorq %%rdx,%%rdx\n" );
-            output( "\txrstor64 (%%r12)\n" );
-            output( "\tmovq %%r11,%%rax\n" );
-        }
-        output( "\tmovq -0x30(%%rbp),%%r15\n" );
-        output( "\tmovq -0x38(%%rbp),%%r14\n" );
-        output( "\tmovq -0x40(%%rbp),%%r13\n" );
-        output( "\tmovq -0x48(%%rbp),%%r12\n" );
-        output_cfi( ".cfi_same_value %%r12" );
-        output( "\tmovq -0x70(%%rbp),%%rdi\n" );
-        output_cfi( ".cfi_same_value %%rdi" );
-        output( "\tmovq -0x78(%%rbp),%%rsi\n" );
-        output_cfi( ".cfi_same_value %%rsi" );
-        output( "\tmovq -0x90(%%rbp),%%rbx\n" );
-        output_cfi( ".cfi_same_value %%rbx" );
-        output( "\tleaq -0x28(%%rbp),%%rsp\n" );
-        output_cfi( ".cfi_def_cfa_register %%rsp" );
-        output_cfi( ".cfi_adjust_cfa_offset 40" );
-        output( "\tmovq (%%rbp),%%rbp\n" );
-        output_cfi( ".cfi_same_value %%rbp" );
-        output( "\tiretq\n" );
-        output( "3:\tmovl $0x%x,%%eax\n", invalid_param );
-        output( "\tjmp 2b\n" );
-        break;
-    case CPU_ARM:
-        output( "\tpush {r5-r11,lr}\n" );
-        output( "\tadd r6, sp, #40\n" );  /* stack parameters */
-        output( "\tldr r5, 6f+8\n" );
-        output( "\tcmp r4, r5\n" );
-        output( "\tbcs 5f\n" );
-        output( "\tsub sp, sp, #8\n" );
-        output( "\tmrc p15, 0, r7, c13, c0, 2\n" ); /* NtCurrentTeb() */
-        output( "\tadd r7, #0x1d8\n" );  /* arm_thread_data()->syscall_frame */
-        output( "\tmrs ip, CPSR\n" );
-        output( "\tbfi ip, lr, #5, #1\n" );  /* set thumb bit */
-        output( "\tstr ip, [sp, #4]\n" );
-        output( "\tstr sp, [r7]\n" );  /* syscall frame */
-        output( "\tldr r5, 6f+4\n");
-        if (UsePIC) output( "1:\tadd r5, pc\n");
-        output( "\tldrb r5, [r5, r4]\n" );  /* syscall args */
-        output( "\tsubs r5, #16\n" );   /* first 4 args are in registers */
-        output( "\tble 3f\n" );
-        output( "\tsub ip, sp, r5\n" );
-        output( "\tand ip, #~7\n" );
-        output( "\tmov sp, ip\n" );
-        output( "2:\tsubs r5, r5, #4\n" );
-        output( "\tldr ip, [r6, r5]\n" );
-        output( "\tstr ip, [sp, r5]\n" );
-        output( "\tbgt 2b\n" );
-        output( "3:\tldr r5, 6f\n");
-        if (UsePIC) output( "4:\tadd r5, pc\n");
-        output( "\tldr ip, [r5, r4, lsl #2]\n");  /* syscall table */
-        output( "\tblx ip\n");
-        output( "\tmov ip, #0\n" );
-        output( "\tstr ip, [r7]\n" );
-        output( "\tsub ip, r6, #40\n" );
-        output( "\tmov sp, ip\n" );
-        output( "\tpop {r5-r11,pc}\n" );
-        output( "5:\tldr r0, 6f+12\n" );
-        output( "\tpop {r5-r11,pc}\n" );
-        if (UsePIC)
-        {
-            output( "6:\t.long .Lsyscall_table-4b-%u\n", thumb_mode ? 4 : 8 );
-            output( "\t.long .Lsyscall_args-1b-%u\n", thumb_mode ? 4 : 8 );
-        }
-        else
-        {
-            output( "6:\t.long .Lsyscall_table\n" );
-            output( "\t.long .Lsyscall_args\n" );
-        }
-        output( "\t.long %u\n", count );
-        output( "\t.long 0x%x\n", invalid_param );
-        break;
-    case CPU_ARM64:
-        output( "\tcmp x8, %u\n", count );
-        output( "\tbcs 3f\n" );
-        output( "\tstp x29, x30, [sp,#-160]!\n" );
-        output_cfi( "\t.cfi_def_cfa_offset 160\n" );
-        output_cfi( "\t.cfi_offset 29, -160\n" );
-        output_cfi( "\t.cfi_offset 30, -152\n" );
-        output( "\tmov x29, sp\n" );
-        output_cfi( "\t.cfi_def_cfa_register 29\n" );
-        output( "\tstp x27, x28, [sp, #144]\n" );
-        output_cfi( "\t.cfi_offset 27, -16\n" );
-        output_cfi( "\t.cfi_offset 28, -8\n" );
-        output( "\tstp x25, x26, [sp, #128]\n" );
-        output_cfi( "\t.cfi_offset 25, -32\n" );
-        output_cfi( "\t.cfi_offset 26, -24\n" );
-        output( "\tstp x23, x24, [sp, #112]\n" );
-        output_cfi( "\t.cfi_offset 23, -48\n" );
-        output_cfi( "\t.cfi_offset 24, -40\n" );
-        output( "\tstp x21, x22, [sp, #96]\n" );
-        output_cfi( "\t.cfi_offset 21, -64\n" );
-        output_cfi( "\t.cfi_offset 22, -56\n" );
-        output( "\tstp x19, x20, [sp, #80]\n" );
-        output_cfi( "\t.cfi_offset 19, -80\n" );
-        output_cfi( "\t.cfi_offset 20, -72\n" );
-        output( "\tstp x6, x7, [sp, #64]\n" );
-        output( "\tstp x4, x5, [sp, #48]\n" );
-        output( "\tstp x2, x3, [sp, #32]\n" );
-        output( "\tstp x0, x1, [sp, #16]\n" );
-        output( "\tmov x20, x8\n" );
-        output( "\tbl %s\n", asm_name("NtCurrentTeb") );
-        output( "\tadd x19, x0, #0x2f8\n" );  /* arm64_thread_data()->syscall_frame */
-        output( "\tstr x29, [x19]\n" );
-        output( "\tldp x0, x1, [sp, #16]\n" );
-        output( "\tldp x2, x3, [sp, #32]\n" );
-        output( "\tldp x4, x5, [sp, #48]\n" );
-        output( "\tldp x6, x7, [sp, #64]\n" );
-        output( "\tadrp x16, %s\n", arm64_page(".Lsyscall_args") );
-        output( "\tadd x16, x16, #%s\n", arm64_pageoff(".Lsyscall_args") );
-        output( "\tldrb w9, [x16, x20]\n" );
-        output( "\tsubs x9, x9, #64\n" );
-        output( "\tbls 2f\n" );
-        output( "\tadd x11, x29, #176\n" );
-        output( "\tsub sp, sp, x9\n" );
-        output( "\ttbz x9, #3, 1f\n" );
-        output( "\tsub sp, sp, #8\n" );
-        output( "1:\tsub x9, x9, #8\n" );
-        output( "\tldr x10, [x11, x9]\n" );
-        output( "\tstr x10, [sp, x9]\n" );
-        output( "\tcbnz x9, 1b\n" );
-        output( "2:\tadrp x16, %s\n", arm64_page(".Lsyscall_table") );
-        output( "\tadd x16, x16, #%s\n", arm64_pageoff(".Lsyscall_table") );
-        output( "\tldr x16, [x16, x20, lsl 3]\n" );
-        output( "\tblr x16\n" );
-        output( "\tmov sp, x29\n" );
-        output( "\tstr xzr, [x19]\n" );
-        output( "\tldp x19, x20, [sp, #80]\n" );
-        output( "\tldp x21, x22, [sp, #96]\n" );
-        output( "\tldp x23, x24, [sp, #112]\n" );
-        output( "\tldp x25, x26, [sp, #128]\n" );
-        output( "\tldp x27, x28, [sp, #144]\n" );
-        output( "\tldp x29, x30, [sp], #160\n" );
-        output( "\tret\n" );
-        output( "3:\tmov x0, #0x%x\n", invalid_param & 0xffff0000 );
-        output( "\tmovk x0, #0x%x\n", invalid_param & 0x0000ffff );
-        output( "\tret\n" );
-        break;
-    default:
-        assert(0);
-    }
-    output_cfi( ".cfi_endproc" );
-    output_function_size( symbol );
-}
-
-
 /* output the functions for system calls */
 void output_syscalls( DLLSPEC *spec )
 {
     int i, count;
     ORDDEF **syscalls = NULL;
 
+    if (unix_lib) return;
+
     for (i = count = 0; i < spec->nb_entry_points; i++)
     {
         ORDDEF *odp = &spec->entry_points[i];
@@ -1838,40 +1441,12 @@ void output_syscalls( DLLSPEC *spec )
     output( "\n/* system calls */\n\n" );
     output( "\t.text\n" );
 
-    if (unix_lib)
-    {
-        output_syscall_dispatcher( count, "" );
-
-        switch( target_cpu )
-        {
-        case CPU_x86:
-            output_syscall_dispatcher( count, "_fxsave" );
-            output_syscall_dispatcher( count, "_xsave" );
-            output_syscall_dispatcher( count, "_xsavec" );
-            break;
-        case CPU_x86_64:
-            output_syscall_dispatcher( count, "_xsave" );
-            output_syscall_dispatcher( count, "_xsavec" );
-            break;
-        default:
-            break;
-        }
-
-        output( "\t.data\n" );
-        output( "\t.align %d\n", get_alignment( get_ptr_size() ) );
-        output( ".Lsyscall_table:\n" );
-        for (i = 0; i < count; i++)
-            output( "\t%s %s\n", get_asm_ptr_keyword(), asm_name( get_link_name( syscalls[i] )));
-        output( ".Lsyscall_args:\n" );
-        for (i = 0; i < count; i++)
-            output( "\t.byte %u\n", get_args_size( syscalls[i] ));
-        return;
-    }
-
     for (i = 0; i < count; i++)
     {
         ORDDEF *odp = syscalls[i];
         const char *name = get_link_name(odp);
+        unsigned int id = (spec->syscall_table << 12) + i;
+
         output( "\t.align %d\n", get_alignment(16) );
         output( "\t%s\n", func_declaration(name) );
         output( "%s\n", asm_globl(name) );
@@ -1883,12 +1458,12 @@ void output_syscalls( DLLSPEC *spec )
             {
                 output( "\tcall %s\n", asm_name("__wine_spec_get_pc_thunk_eax") );
                 output( "1:\tmovl %s-1b(%%eax),%%edx\n", asm_name("__wine_syscall_dispatcher") );
-                output( "\tmovl $%u,%%eax\n", i );
+                output( "\tmovl $%u,%%eax\n", id );
                 needs_get_pc_thunk = 1;
             }
             else
             {
-                output( "\tmovl $%u,%%eax\n", i );
+                output( "\tmovl $%u,%%eax\n", id );
                 output( "\tmovl $%s,%%edx\n", asm_name("__wine_syscall") );
             }
             output( "\tcall *%%edx\n" );
@@ -1902,7 +1477,7 @@ void output_syscalls( DLLSPEC *spec )
              * validate that instruction, we can just put a jmp there instead. */
             output( "\t.byte 0x4c,0x8b,0xd1\n" ); /* movq %rcx,%r10 */
             output( "\t.byte 0xb8\n" );           /* movl $i,%eax */
-            output( "\t.long %u\n", i );
+            output( "\t.long %u\n", id );
             output( "\t.byte 0xf6,0x04,0x25,0x08,0x03,0xfe,0x7f,0x01\n" ); /* testb $1,0x7ffe0308 */
             output( "\t.byte 0x75,0x03\n" );      /* jne 1f */
             output( "\t.byte 0x0f,0x05\n" );      /* syscall */
@@ -1922,33 +1497,17 @@ void output_syscalls( DLLSPEC *spec )
             output( "\tret\n" );
             break;
         case CPU_ARM:
-            output( "\tpush {r4,lr}\n" );
-            output( "\tmov r4, #%u\n", i );
-            if (UsePIC)
-            {
-                output( "\tldr ip, 2f\n");
-                output( "1:\tadd ip, pc\n" );
-            }
-            else
-            {
-                output( "\tmovw ip, :lower16:%s\n", asm_name("__wine_syscall_dispatcher") );
-                output( "\tmovt ip, :upper16:%s\n", asm_name("__wine_syscall_dispatcher") );
-            }
-            output( "\tldr ip, [ip]\n");
-            output( "\tblx ip\n");
-            output( "\tpop {r4,pc}\n" );
-            if (UsePIC) output( "2:\t.long %s-1b-%u\n", asm_name("__wine_syscall_dispatcher"), thumb_mode ? 4 : 8 );
+            output( "\tpush {r0-r3}\n" );
+            output( "\tmovw ip, #%u\n", id );
+            output( "\tmov r3, lr\n" );
+            output( "\tbl %s\n", asm_name("__wine_syscall") );
+            output( "\tadd sp, #16\n" );
+            output( "\tbx lr\n" );
             break;
         case CPU_ARM64:
-            output( "\tstp x29, x30, [sp,#-16]!\n" );
-            output_cfi( "\t.cfi_def_cfa_offset 16\n" );
-            output_cfi( "\t.cfi_offset 29, -16\n" );
-            output_cfi( "\t.cfi_offset 30, -8\n" );
-            output( "\tmov x8, #%u\n", i );
-            output( "\tadrp x16, %s\n", arm64_page( asm_name("__wine_syscall_dispatcher") ) );
-            output( "\tldr x16, [x16, #%s]\n", arm64_pageoff( asm_name("__wine_syscall_dispatcher") ) );
-            output( "\tblr x16\n");
-            output( "\tldp x29, x30, [sp], #16\n" );
+            output( "\tmov x8, #%u\n", id );
+            output( "\tmov x9, x30\n" );
+            output( "\tbl %s\n", asm_name("__wine_syscall" ));
             output( "\tret\n" );
             break;
         default:
@@ -1958,18 +1517,52 @@ void output_syscalls( DLLSPEC *spec )
         output_function_size( name );
     }
 
-    if (target_cpu == CPU_x86 && !UsePIC)
+    switch (target_cpu)
     {
+    case CPU_x86:
+        if (UsePIC) break;
         output( "\t.align %d\n", get_alignment(16) );
         output( "\t%s\n", func_declaration("__wine_syscall") );
         output( "%s:\n", asm_name("__wine_syscall") );
         output( "\tjmp *(%s)\n", asm_name("__wine_syscall_dispatcher") );
         output_function_size( "__wine_syscall" );
+        break;
+    case CPU_ARM:
+        output( "\t.align %d\n", get_alignment(16) );
+        output( "\t%s\n", func_declaration("__wine_syscall") );
+        output( "%s:\n", asm_name("__wine_syscall") );
+        if (UsePIC)
+        {
+            output( "\tldr r0, 2f\n");
+            output( "1:\tadd r0, pc\n" );
+        }
+        else
+        {
+            output( "\tmovw r0, :lower16:%s\n", asm_name("__wine_syscall_dispatcher") );
+            output( "\tmovt r0, :upper16:%s\n", asm_name("__wine_syscall_dispatcher") );
+        }
+        output( "\tldr r0, [r0]\n");
+        output( "\tbx r0\n");
+        if (UsePIC) output( "2:\t.long %s-1b-%u\n", asm_name("__wine_syscall_dispatcher"), thumb_mode ? 4 : 8 );
+        output_function_size( "__wine_syscall" );
+        break;
+    case CPU_ARM64:
+        output( "\t.align %d\n", get_alignment(16) );
+        output( "\t%s\n", func_declaration("__wine_syscall") );
+        output( "%s:\n", asm_name("__wine_syscall") );
+        output( "\tadrp x16, %s\n", arm64_page( asm_name("__wine_syscall_dispatcher") ) );
+        output( "\tldr x16, [x16, #%s]\n", arm64_pageoff( asm_name("__wine_syscall_dispatcher") ) );
+        output( "\tbr x16\n");
+        output_function_size( "__wine_syscall" );
+    default:
+        break;
     }
     output( "\t.data\n" );
     output( "\t.align %d\n", get_alignment( get_ptr_size() ) );
     output( "%s\n", asm_globl("__wine_syscall_dispatcher") );
     output( "\t%s 0\n", get_asm_ptr_keyword() );
+    output( "\t.short %u\n", count );
+    for (i = 0; i < count; i++) output( "\t.byte %u\n", get_args_size( syscalls[i] ));
 }
 
 
