@@ -49,6 +49,7 @@ int unwind_tables = 0;
 int use_msvcrt = 0;
 int unix_lib = 0;
 int safe_seh = 0;
+int prefer_native = 0;
 
 #ifdef __i386__
 enum target_cpu target_cpu = CPU_x86;
@@ -66,6 +67,8 @@ enum target_cpu target_cpu = CPU_ARM64;
 
 #ifdef __APPLE__
 enum target_platform target_platform = PLATFORM_APPLE;
+#elif defined(__linux__)
+enum target_platform target_platform = PLATFORM_LINUX;
 #elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 enum target_platform target_platform = PLATFORM_FREEBSD;
 #elif defined(__sun)
@@ -131,6 +134,7 @@ static const struct
 {
     { "macos",       PLATFORM_APPLE },
     { "darwin",      PLATFORM_APPLE },
+    { "linux",       PLATFORM_LINUX },
     { "freebsd",     PLATFORM_FREEBSD },
     { "solaris",     PLATFORM_SOLARIS },
     { "mingw32",     PLATFORM_MINGW },
@@ -198,6 +202,15 @@ static void set_subsystem( const char *subsystem, DLLSPEC *spec )
     free( str );
 }
 
+/* set the syscall table id */
+static void set_syscall_table( const char *id, DLLSPEC *spec )
+{
+    int val = atoi( id );
+
+    if (val < 0 || val > 3) fatal_error( "Invalid syscall table id '%s', must be 0-3\n", id );
+    spec->syscall_table = val;
+}
+
 /* set the target CPU and platform */
 static void set_target( const char *target )
 {
@@ -294,10 +307,12 @@ static const char usage_str[] =
 "       --nxcompat=y|n        Set the NX compatibility flag (default: yes)\n"
 "   -N, --dll-name=DLLNAME    Set the DLL name (default: from input file name)\n"
 "   -o, --output=NAME         Set the output file name (default: stdout)\n"
+"       --prefer-native       Set the flag to prefer loading native at run time\n"
 "   -r, --res=RSRC.RES        Load resources from RSRC.RES\n"
 "       --safeseh             Mark object files as SEH compatible\n"
 "       --save-temps          Do not delete the generated intermediate files\n"
 "       --subsystem=SUBSYS    Set the subsystem (one of native, windows, console, wince)\n"
+"       --syscall-table=ID    Set the syscall table id (between 0 and 3)\n"
 "   -u, --undefined=SYMBOL    Add an undefined reference to SYMBOL when linking\n"
 "   -v, --verbose             Display the programs invoked\n"
 "       --version             Print the version and exit\n"
@@ -329,11 +344,13 @@ enum long_options_values
     LONG_OPT_LDCMD,
     LONG_OPT_NMCMD,
     LONG_OPT_NXCOMPAT,
+    LONG_OPT_PREFER_NATIVE,
     LONG_OPT_RESOURCES,
     LONG_OPT_SAFE_SEH,
     LONG_OPT_SAVE_TEMPS,
     LONG_OPT_STATICLIB,
     LONG_OPT_SUBSYSTEM,
+    LONG_OPT_SYSCALL_TABLE,
     LONG_OPT_VERSION
 };
 
@@ -356,10 +373,12 @@ static const struct option long_options[] =
     { "ld-cmd",        1, 0, LONG_OPT_LDCMD },
     { "nm-cmd",        1, 0, LONG_OPT_NMCMD },
     { "nxcompat",      1, 0, LONG_OPT_NXCOMPAT },
+    { "prefer-native", 0, 0, LONG_OPT_PREFER_NATIVE },
     { "resources",     0, 0, LONG_OPT_RESOURCES },
     { "safeseh",       0, 0, LONG_OPT_SAFE_SEH },
     { "save-temps",    0, 0, LONG_OPT_SAVE_TEMPS },
     { "subsystem",     1, 0, LONG_OPT_SUBSYSTEM },
+    { "syscall-table", 1, 0, LONG_OPT_SYSCALL_TABLE },
     { "version",       0, 0, LONG_OPT_VERSION },
     /* aliases for short options */
     { "target",        1, 0, 'b' },
@@ -575,6 +594,10 @@ static char **parse_options( int argc, char **argv, DLLSPEC *spec )
         case LONG_OPT_SAFE_SEH:
             safe_seh = 1;
             break;
+        case LONG_OPT_PREFER_NATIVE:
+            prefer_native = 1;
+            spec->dll_characteristics |= IMAGE_DLLCHARACTERISTICS_PREFER_NATIVE;
+            break;
         case LONG_OPT_RESOURCES:
             set_exec_mode( MODE_RESOURCES );
             break;
@@ -584,6 +607,9 @@ static char **parse_options( int argc, char **argv, DLLSPEC *spec )
         case LONG_OPT_SUBSYSTEM:
             set_subsystem( optarg, spec );
             break;
+        case LONG_OPT_SYSCALL_TABLE:
+            set_syscall_table( optarg, spec );
+            break;
         case LONG_OPT_VERSION:
             printf( "winebuild version " PACKAGE_VERSION "\n" );
             exit(0);
